#! /usr/bin/env python
# -*- coding: utf-8 -*-
""" Example with use of pyqt read 3d image
"""

# import funkcí z jiného adresáře
import logging
import sys

from PyQt4 import QtGui
from PyQt4.QtGui import QGridLayout, QLabel, \
    QPushButton, QLineEdit, QApplication

logger = logging.getLogger(__name__)
import argparse

from io3d.datareaderqt import DataReaderWidget

def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    parser.add_argument(
        '-i', '--inputfile',
        default=None,
        required=True,
        help='input file'
    )
    parser.add_argument(
        '-sn', '--seriesnumber',
        default=None,
        help='seriesnumber'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)

    data3d, metadata = read(args.inputfile, series_number = args.seriesnumber)


def show(data3dw):
    data3d = data3dw.datap["data3d"]
    import sed3
    ed = sed3.sed3qt(data3d)
    # ed = sed3.se
    # ed.show()
    ed.exec_()

if __name__ == "__main__":
    # main()

    app = QtGui.QApplication(sys.argv)

    # w = QtGui.QWidget()
    # w = DictEdit(dictionary={'jatra':2, 'ledviny':7})
    w = DataReaderWidget(loaddir="~", loadfiledir="~",
                         cachefile="~/cache.yaml",
                         after_function=None, before_function=None)
    w.resize(250, 150)
    w.move(300, 300)
    w.setWindowTitle('io3dQtWidget')
    w.show()

    app.exec_()
    data3d = w.datap["data3d"]

    import sed3
    ed = sed3.sed3(data3d)
    ed.show()



#! /usr/bin/python
# -*- coding: utf-8 -*-
"""
Module is used for visualization of segmentation stored in pkl, dcm and other files.
"""

import logging
logger = logging.getLogger(__name__)
import os.path
import sys
import argparse
import numpy as np
import zipfile
import glob
import os.path as op
import io3d
from . import cachefile as cachef
# if sys.version_info < (3, 0):
#     import urllib as urllibr
# else:
#     import urllib.request as urllibr


# you can get hash from command line with:
#  python imtools/sample_data.py -v sliver_training_001
# local_dir = "~/data/medical/orig/"
local_dir = "~/data/"
# vessels.pkl nejprve vytvoří prázný adresář s názvem vessels.pkl, pak jej při rozbalování zase smaže
__url_home = "http://home.zcu.cz/~mjirik/lisa/testdata/sample-extra-data/"
__url_server = "http://147.228.240.61/queetech/"
__url_server = "http://home.zcu.cz/~mjirik/lisa/"
data_urls = {
    "head": [__url_server + "sample-data/head.zip", "89e9b60fd23257f01c4a1632ff7bb800", "matlab"],
    "jatra_06mm_jenjatra": [__url_server + "sample-data/jatra_06mm_jenjatra.zip", None, "jatra_06mm_jenjatra/*.dcm"],
    "jatra_5mm": [__url_server + "sample-data/jatra_5mm.zip", '1b9039ffe1ff9af9caa344341c8cec03', "jatra_5mm/*.dcm"],
    "exp": [__url_server + "sample-data/exp.zip", '74f2c10b17b6bd31bd03662df6cf884d'],
    "sliver_training_001": [__url_server + "sample-data/sliver_training_001.zip", "d64235727c0adafe13d24bfb311d1ed0",
                            "liver*001.*"],
    "volumetrie": [__url_server + "sample-data/volumetrie.zip", "6b2a2da67874ba526e2fe00a78dd19c9"],
    "vessels.pkl": [__url_server + "sample-data/vessels.pkl.zip", "698ef2bc345bb616f8d4195048538ded"],
    "biodur_sample": [__url_server + "sample-data/biodur_sample.zip", "d459dd5b308ca07d10414b3a3a9000ea"],
    "gensei_slices": [__url_server + "sample-data/gensei_slices.zip", "ef93b121add8e4a133bb086e9e6491c9"],
    "exp_small": [__url_server + "sample-data/exp_small.zip", "0526ba8ea363fe8b5227f5807b7aaca7"],
    "vincentka": [__url_server + "sample-data/vincentka.zip", "a30fdabaa39c5ce032a3223ed30b88e3"],
    "vincentka_sample": [__url_server + "sample-data/vincentka_sample.zip"],
    "donut": __url_server + "sample-data/donut.zip",
    # "io3d_sample_data": [__url_server + "sample-extra-data/io3d_sample_data.zip"],
    "io3d_sample_data": [__url_server + "sample-data/io3d_sample_data.zip"],
    "lisa": {"package": ["donut", "vincentka_sample", "exp_small", "gensei_slices",
                         "biodur_sample", "vessels.pkl", "sliver_training_001", "jatra_5mm",
                         "head", "volumetrie"]},
    # "3Dircadb1": ["http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.zip", None, None, "ircad/*[!p]/*[!pfg]"],
    "3Dircadb1" : {"package": [
        "3Dircadb1.1", "3Dircadb1.2", "3Dircadb1.3", "3Dircadb1.4", "3Dircadb1.5", "3Dircadb1.6", "3Dircadb1.7",
        "3Dircadb1.8", "3Dircadb1.9", "3Dircadb1.10", "3Dircadb1.11", "3Dircadb1.12", "3Dircadb1.13",
        "3Dircadb1.14", "3Dircadb1.15", "3Dircadb1.16", "3Dircadb1.17", "3Dircadb1.18", "3Dircadb1.19", "3Dircadb1.20",
    ]},
    "3Dircadb1.1": ["http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.1.zip", "8ab16d83bfb58b790b9ca18f81401cdf"],
    "3Dircadb1.2": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.2.zip",
    "3Dircadb1.3": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.3.zip",
    "3Dircadb1.4": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.4.zip",
    "3Dircadb1.5": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.5.zip",
    "3Dircadb1.6": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.6.zip",
    "3Dircadb1.7": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.7.zip",
    "3Dircadb1.8": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.8.zip",
    "3Dircadb1.9": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.9.zip",
    "3Dircadb1.10": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.10.zip",
    "3Dircadb1.11": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.11.zip",
    "3Dircadb1.12": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.12.zip",
    "3Dircadb1.13": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.13.zip",
    "3Dircadb1.14": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.14.zip",
    "3Dircadb1.15": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.15.zip",
    "3Dircadb1.16": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.16.zip",
    "3Dircadb1.17": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.17.zip",
    "3Dircadb1.18": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.18.zip",
    "3Dircadb1.19": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.19.zip",
    "3Dircadb1.20": "http://ircad.fr/softwares/3Dircadb/3Dircadb1/3Dircadb1.20.zip",
    "CMU-1": ["http://openslide.cs.cmu.edu/download/openslide-testdata/Hamamatsu/CMU-1.ndpi",
              "1f9df18b8dc6403cd2434bfd3cf20c7f",
              "CMU-1.ndpi"],
    "CMU-1-annotation": [
        __url_server + "sample-data/CMU-1.ndpi.ndpa",
        "f2f70fe432bd9dc3c252540313b92df0",
        "CMU-1.ndpi.ndpa"
    ],

    # není nutné pole, stačí jen string
    # "exp_small": "http://147.228.240.61/queetech/sample-data/exp_small.zip",
}
# cachefile = "~/io3d_cache.yaml"


def join_path(*path_to_join, **kwargs):
    """Join input path to sample data path (usually in ~/lisa_data)

    :param path_to_join: one or more paths
    :param get_root: return dataset root path. If false, the path would be into "medical/orig"
    :return: joined path
    """
    if "get_root" in kwargs:
        get_root = kwargs["get_root"]
    else:
        # default value
        get_root = False
    sdp = dataset_path(get_root=get_root)
    pth = os.path.join(sdp, *path_to_join)
    logger.debug('sample_data_path' + str(sdp))
    logger.debug('path ' + str(pth))
    return pth


def set_dataset_path(path, cache=None, cachefile="~/.io3d_cache.yaml"):
    """Sets path to dataset. Warning: function with side effects!

    :param path: path you want to store dataset
    :param cache: CacheFile object
    :param cachefile: default '~/.io3d_cache.yaml'
    """
    if cachefile is not None:
        cache = cachef.CacheFile(cachefile)
    cache.update("local_dataset_dir", path)


def dataset_path(cache=None, cachefile="~/.io3d_cache.yaml", get_root=False):
    """Get dataset path.

    :param cache: CacheFile object
    :param cachefile: cachefile path, default '~/.io3d_cache.yaml'
    :return: path to dataset
    """
    local_data_dir = local_dir

    if cachefile is not None:
        cache = cachef.CacheFile(cachefile)
        # cache.update('local_dataset_dir', head)
    if cache is not None:
        local_data_dir = cache.get_or_save_default('local_dataset_dir', local_dir)

    if get_root:
        local_data_dir
    else:
        logger.warning("Parameter")
        local_data_dir = op.join(local_data_dir, "medical", "orig")

    return op.expanduser(local_data_dir)

# def get_sample_data():
#     keys = imtools.sample_data.data_urls.keys()
#     imtools.sample_data.get_sample_data(keys, sample_data_path())


# noinspection PyUnboundLocalVariable
def get_dataset_meta(label):
    """Gives you metadata for dataset chosen via 'label' param

    :param label: label = key in data_url dict (that big dict containing all possible datasets)
    :return: tuple (data_url, url, expected_hash, hash_path, fnpattern)
    """
    data_url = data_urls[label]
    if type(data_url) == str:
        # back compatibility
        data_url = [data_url]
    if type(data_url) == list:
        data_url.extend([None, None, None])
        data_url = data_url[:4]
        url, expected_hash, hash_path, fnpattern = data_url
        if hash_path is None:
            hash_path = label
        if fnpattern is None and hash_path is not None:
            fnpattern = hash_path
    # elif type(data_url) == dict:
    return data_url, url, expected_hash, hash_path, fnpattern


# noinspection PyTypeChecker
def _expand_dataset_packages(dataset_label_dict):
    """Returns list of possible packages contained in dataset, in case the dataset is multi dataset, eg. 'lisa'.

    In case the param is not pointing to multidataset returns only that label in a list.

    :param str dataset_label_dict: label of multi dataset
    :return: list of labels
    """
    new_dataset_label_dict = []
    for label in dataset_label_dict:
        dataset_metadata = data_urls[label]
        if type(dataset_metadata) == dict and "package" in dataset_metadata:
            new_dataset_label_dict.extend(dataset_metadata["package"])
        else:
            new_dataset_label_dict.append(label)
    return new_dataset_label_dict


def download(dataset_label=None, destination_dir=None, dry_run=False):
    """Download sample data by data label. Warning: function with side effect!

    Labels can be listed by sample_data.data_urls.keys(). Returns downloaded files.

    :param dataset_label: label of data. If it is set to None, all data are downloaded
    :param destination_dir: output dir for data
    :param dry_run: runs function without downloading anything
    """
    if destination_dir is None:
        destination_dir = op.join(dataset_path(get_root=True), "medical", "orig")

    destination_dir = op.expanduser(destination_dir)
    logger.info("destination dir: {}".format(destination_dir))

    if not op.exists(destination_dir):
        logger.debug("creating directory {}".format(destination_dir))
        os.makedirs(destination_dir)

    if dataset_label is None:
        dataset_label = data_urls.keys()

    if type(dataset_label) == str:
        dataset_label = [dataset_label]

    dataset_label = _expand_dataset_packages(dataset_label)

    for label in dataset_label:
        # make all data:url have length 3
        data_url, url, expected_hash, hash_path, fnpattern = get_dataset_meta(label)

        if hash_path is None:
            hash_path = label

        path_to_hash = os.path.join(destination_dir, hash_path)
        try:
            computed_hash = checksum(path_to_hash)
        except Exception as e:
            # there is probably no checksumdir module
            logger.warning(e)
            logger.warning("problem with sample_data.checksum()")
            computed_hash = None

        logger.info("dataset: '" + label + "'")
        logger.info("path to hash: {}".format(path_to_hash))
        logger.info("expected hash: '" + str(expected_hash) + "'")
        logger.info("computed hash: '" + str(computed_hash) + "'")
        if (computed_hash is not None) and (expected_hash == computed_hash):
            logger.info("match ok - no download needed")
        else:
            logger.info("downloading")
            if not dry_run:
                downzip(url, destination=destination_dir)
                logger.info("finished")
                downloaded_hash = checksum(os.path.join(destination_dir, hash_path))
                logger.info("downloaded hash: '" + str(downloaded_hash) + "'")
                if downloaded_hash != expected_hash:
                    logger.warning("downloaded hash is different from expected hash\n" +
                                   "expected hash: '" + str(expected_hash) + "'\n" +
                                   "downloaded hash: '" + str(downloaded_hash) + "'\n")
            else:
                logger.debug("dry run")


# NOTE(mareklovci): I suppose, this isn't working at all
def get_old(dataset_label, data_id, destination_dir=None):
    """Get the 3D data from specified dataset with specified id.

    Download data if necessary.

    :param dataset_label:
    :param data_id: integer or wildcards file pattern
    :param destination_dir:
    :return:
    """
    # TODO implement
    if destination_dir is None:
        destination_dir = op.join(dataset_path(get_root=True), "medical", "orig")

    destination_dir = op.expanduser(destination_dir)
    data_url, url, expected_hash, hash_path, fnpattern = get_dataset_meta(dataset_label)
    paths = glob.glob(os.path.join(destination_dir, fnpattern))
    paths.sort()
    import fnmatch
    print(paths)
    print(data_id)
    pathsf = fnmatch.filter(paths, data_id)
    print(pathsf)
    datap = io3d.read(pathsf[0], dataplus_format=True)
    return datap


# NOTE(mareklovci - 2018_05_14): work in progress
# noinspection PyUnusedLocal
def get(dataset_label, series_number=None, *args, **kwargs):
    """

    :param dataset_label: label from data_urls
    :param series_number: Series identification in study.
    :param args:
    :param kwargs:
    :return:
    """

    # relative path in the datasets
    relative_path_extracted_from_data_urls = ""
    datapath = join_path(relative_path_extracted_from_data_urls, "medical", "data", get_root=True)
    # read 3D data from datapath
    datap = io3d.read(datapath, series_number=series_number, dataplus_format=True, *args, **kwargs)
    return datap


# noinspection PyProtectedMember
def checksum(path, hashfunc='md5'):
    """Return checksum of files given by path.

    Wildcards can be used in check sum. Function is strongly dependent on checksumdir package by 'cakepietoast'.

    :param path: path of files to get hash from
    :param hashfunc: function used to get hash, default 'md5'
    :return: (str) hash of the file/files given by path
    """
    import checksumdir
    hash_func = checksumdir.HASH_FUNCS.get(hashfunc)
    if not hash_func:
        raise NotImplementedError('{} not implemented.'.format(hashfunc))

    if os.path.isdir(path):
        return checksumdir.dirhash(path, hashfunc=hashfunc)

    hashvalues = []
    path_list = glob.glob(path)
    logger.debug("path_list " + str(path_list))
    for path in path_list:
        if os.path.isfile(path):
            hashvalues.append(checksumdir._filehash(path, hashfunc=hash_func))
    logger.debug("one hash per file: {}".format(str(hashvalues)))
    checksum_hash = checksumdir._reduce_hash(hashvalues, hashfunc=hash_func)
    logger.debug("total hash: {}".format(str(checksum_hash)))
    return checksum_hash


def generate_donut():
    """Generate donut like shape with stick inside

    :return: dict {'data3d': '', 'segmentation': '', 'voxelsize_mm': ''}
    """
    segmentation = np.zeros([20, 30, 40])
    # generate test data
    segmentation[6:10, 7:24, 10:37] = 1
    segmentation[6:10, 7, 10] = 0
    segmentation[6:10, 23, 10] = 0
    segmentation[6:10, 7, 36] = 0
    segmentation[6:10, 23, 36] = 0
    segmentation[2:18, 12:19, 18:28] = 2

    data3d = segmentation * 100 + np.random.random(segmentation.shape) * 30
    voxelsize_mm = [3, 2, 1]

    datap = {
        'data3d': data3d,
        'segmentation': segmentation,
        'voxelsize_mm': voxelsize_mm
    }
    # io3d.write(datap, "donut.pklz")
    return datap


def generate_abdominal(size=100, liver_intensity=100, noise_intensity=20, portal_vein_intensity=130,
                       spleen_intensity=90):
    """Create scaleable artificial abdominal like data. Outputs a cube.

    {0: nothing, 1: liver, 2: portal_vein, 17: spleen}

    :param size: the length of the cube edge
    :param liver_intensity: "luminosity" of liver
    :param noise_intensity: adding noise to data
    :param portal_vein_intensity: "luminosity" of portal vein
    :param spleen_intensity: "luminosity" of spleen
    :return: {'data3d': '', 'segmentation': '', 'voxelsize_mm': '', 'seeds': '', 'slab': ''}
    """
    boundary = int(size / 4)
    voxelsize_mm = [1.0, 1.5, 1.5]
    slab = {
        'liver': 1,
        'porta': 2,
        'spleen': 17
    }

    segmentation = np.zeros([size, size, size], dtype=np.uint8)
    segmentation[boundary:-boundary, boundary:-2*boundary, 2*boundary:-boundary] = slab["liver"]
    segmentation[:, boundary*2:boundary*2+5, boundary*2:boundary*2+5] = slab["porta"]
    segmentation[:, boundary*2:boundary*2+5, boundary*2:boundary*2+5] = slab["porta"]
    segmentation[:, -5:, -boundary:] = 17
    seeds = np.zeros([size, size, size], dtype=np.uint8)
    seeds[
        boundary + 1: boundary + 4,
        boundary + 1: boundary + 4,
        2 * boundary + 1: 2 * boundary + 4
    ] = 1

    noise = (np.random.random(segmentation.shape) * noise_intensity).astype(np.int)
    data3d = np.zeros(segmentation.shape, dtype=np.int)
    data3d[segmentation == 1] = liver_intensity
    data3d[segmentation == 2] = portal_vein_intensity
    data3d[segmentation == 17] = spleen_intensity
    data3d += noise
    datap = {
        'data3d': data3d,
        'segmentation': segmentation,
        'voxelsize_mm': voxelsize_mm,
        'seeds': seeds,
        'slab': slab
    }
    return datap


def generate_round_data(sz=32, offset=0, radius=7, seedsz=3, add_object_without_seeds=False):
    """
    Generate data with two sphere objects.
    :param sz: output data shape is [sz, sz+1, sz+2]
    :param offset:
    :param radius:
    :param seedsz:
    :param add_object_without_seeds: Add also one cube-like object in the corner.
    :return:
    """

    import scipy.ndimage
    #seedsz= int(sz/10)
    space=2
    seeds = np.zeros([sz, sz+1, sz+2], dtype=np.int8)
    xmin = radius + seedsz + offset + 2
    ymin = radius + seedsz + offset + 6
    seeds[offset + 12, xmin + 3:xmin + 7 + seedsz, ymin:ymin+2] = 1
    seeds[offset + 20, xmin + 7:xmin + 12 + seedsz, ymin+5:ymin+7] = 1

    # add temp seed
    if add_object_without_seeds:
        seeds[-3, -3, -3] = 1
    img = np.ones([sz, sz+1, sz+2])
    img = img - seeds

    seeds[
    2:10 + seedsz,
    2:9+ seedsz,
    2:3+ seedsz] = 2

    # remove temo seed
    if add_object_without_seeds:
        seeds[-3, -3, -3] = 0

    img = scipy.ndimage.morphology.distance_transform_edt(img)
    segm = img < radius
    img = (100 * segm + 80 * np.random.random(img.shape)).astype(np.uint8)
    return img, segm, seeds


def generate_synthetic_liver(return_dataplus=False):
    """
    Create synthetic data. There is some liver and porta -like object.
    :return data3d, segmentation, voxelsize_mm, slab, seeds_liver, seeds_porta:
    """
    # data
    slab = {'none': 0, 'liver': 1, 'porta': 2}
    voxelsize_mm = np.array([1.0, 1.0, 1.2])

    segm = np.zeros([80, 256, 250], dtype=np.int16)

    # liver
    segm[30:60, 70:180, 40:190] = slab['liver']

    # porta
    segm[40:45, 120:130, 70:190] = slab['porta']
    segm[41:44, 122:127, 68:70] = slab['porta'] # hack to fix stability of skeleton algorithm
    #
    segm[40:45, 80:130, 100:110] = slab['porta']
    segm[42:44, 77:80, 103:106] = slab['porta'] # hack to fix stability of skeleton algorithm
    # segm[41:44, 78:80, 101:109] = slab['porta']
    # vertical branch under main branch
    segm[40:44, 120:170, 130:135] = slab['porta']

    data3d = np.zeros(segm.shape)
    data3d[segm == slab['liver']] = 146
    data3d[segm == slab['porta']] = 206
    noise = (np.random.normal(0, 10, segm.shape))  # .astype(np.int16)
    data3d = (data3d + noise).astype(np.int16)


    seeds_liver = np.zeros(data3d.shape, np.int8)
    seeds_liver[40:55, 90:120, 70:110] = 1
    seeds_liver[30:45, 190:200, 40:90] = 2

    seeds_porta = np.zeros(data3d.shape, np.int8)
    seeds_porta[40:45, 121:139, 80:95] = 1

    if return_dataplus:
        datap = {
            "data3d": data3d,
            "voxelsize_mm": voxelsize_mm,
            "slab": slab,
            "seeds_liver": seeds_liver,
            "seeds_porta": seeds_porta,
            "segmentation": segm

        }
        return datap
    else:
        return data3d, segm, voxelsize_mm, slab, seeds_liver, seeds_porta


def _get_face2(shape=None, face_r=1.0, smile_r1=0.5, smile_r2=0.7, eye_r=0.2):
    """
    Create 2D binar face
    :param shape:
    :param face_r:
    :param smile_r1:
    :param smile_r2:
    :param eye_r:
    :return:
    """

    # data3d = np.zeros([1,7,7], dtype=np.int16)
    if shape is None:
        shape = [32, 32]

    center = ((np.asarray(shape) - 1) / 2.0)
    r = np.min(center) * face_r

    # np.min(np.asarray(shape) / 2.0)
    # shape = data3d.shape[1:]
    # data3d[center[0], center[1], center[2]] = 1
    x, y = np.meshgrid(range(shape[1]), range(shape[0]))

    head = (x - center[0]) ** 2 + (y - center[1]) ** 2 < r ** 2

    smile = ((x - center[0]) ** 2 + (y - center[1]) ** 2 < (r * smile_r2) ** 2) & (y > (center[1] + 0.3 * r)) & (
                (x - center[0]) ** 2 + (y - center[1]) ** 2 >= (r * smile_r1) ** 2)
    smile
    e1c = center + r * np.array([-0.35, -0.2])
    e2c = center + r * np.array([0.35, -0.2])

    eyes = (x - e1c[0]) ** 2 + (y - e1c[1]) ** 2 <= (r * eye_r) ** 2
    eyes += (x - e2c[0]) ** 2 + (y - e1c[1]) ** 2 <= (r * eye_r) ** 2

    face = head & ~smile & ~eyes
    return face


def generate_face(shape=None, face_r=1.0, smile_r1=0.5, smile_r2=0.7, eye_r=0.2):
    """
    Create 2D or 3D binar data with smile face.

    :param shape: 2D or 3D shape of data
    :param face_r:
    :param smile_r1:
    :param smile_r2:
    :param eye_r:
    :return: binar ndarray
    """
    # TODO add axis (ax=0)
    if shape is None:
        shape = [32, 32]
    nd = len(shape)
    if nd == 2:
        sh2 = shape
    else:
        sh2 = shape[1:]
    fc2 = _get_face2(sh2, face_r=face_r, smile_r1=smile_r1, smile_r2=smile_r2, eye_r=eye_r)

    if nd == 2:
        return fc2
    else:
        fc3 = np.zeros(shape)
        for i in range(fc3.shape[0]):
            fc3[i, :, :] = fc2
        return fc3

def sliver_reader(filename_end_mask="*[0-9].mhd", sliver_reference_dir="~/data/medical/orig/sliver07/training/",
                  read_orig=True, read_seg=False):
    """Generator for reading sliver data from directory structure.

    :param filename_end_mask: file selection can be controlled with this parameter
    :param sliver_reference_dir: directory with sliver .mhd and .raw files
    :param read_orig: read image data if is set True
    :param read_seg: read segmentation data if is set True
    :return: tuple (numeric_label, vs_mm, oname, orig_data, rname, ref_data)
    """
    sliver_reference_dir = op.expanduser(sliver_reference_dir)
    orig_fnames = glob.glob(sliver_reference_dir + "*orig" + filename_end_mask)
    ref_fnames = glob.glob(sliver_reference_dir + "*seg" + filename_end_mask)

    orig_fnames.sort()
    ref_fnames.sort()
    for i in range(0, len(orig_fnames)):
        oname = orig_fnames[i]
        rname = ref_fnames[i]
        vs_mm = None
        ref_data = None
        orig_data = None
        if read_orig:
            orig_data, metadata = io3d.datareader.read(oname, dataplus_format=False)
            vs_mm = metadata['voxelsize_mm']
        if read_seg:
            ref_data, metadata = io3d.datareader.read(rname, dataplus_format=False)
            vs_mm = metadata['voxelsize_mm']

        import re
        numeric_label = re.search(r'.*g(\d+)', oname).group(1)
        out = (numeric_label, vs_mm, oname, orig_data, rname, ref_data)
        yield out


def remove(local_file_name):
    """Function attempts to remove file, if failure occures -> print exception

    :param local_file_name: name of file to remove
    """
    try:
        os.remove(local_file_name)
    except Exception as e:
        print("Cannot remove file '" + local_file_name + "'. Please remove it manually.")
        print(e)


def downzip(url, destination='./sample_data/'):
    """Download, unzip and delete. Warning: function with strong side effects!

    Returns downloaded data.

    :param str url: url from which data should be donloaded
    :param destination: destination to which data should be downloaded
    """

    # url = "http://147.228.240.61/queetech/sample-data/jatra_06mm_jenjatra.zip"
    logmsg = "downloading from '" + url + "' to '" + destination + "'"
    print(logmsg)
    logger.info(logmsg)
    tmp_filename = url[url.rfind("/") + 1:]
    # tmp_filename = "tmp.zip"
    # urllibr.urlretrieve(url, zip_file_name)
    from . import network
    network.download_file(url, destination, filename=tmp_filename)
    zip_file_name = os.path.join(destination, tmp_filename)
    base, ext = op.splitext(tmp_filename)
    print(ext)
    print(tmp_filename)
    if ext in (".zip"):
        unzip_recursive(zip_file_name)
    # unzip_one(local_file_name)

# def unzip_all(path):
#     """ Unzip all .zip files packed in other .zip in path recusively.
#
#     :param path:
#     :return:
#     """
#
#     ziplist = glob.glob(op.join(path, '*.zip'))
#     while len(ziplist) > 0:
#     # for local_file_name in ziplist:
#         local_file_name = ziplist[0]
#         unzip_one(local_file_name)
#         ziplist = glob.glob(op.join(path, '*.zip'))


def unzip_one(local_file_name):
    """Unzips one file and deletes it. Warning: function with side effects!

    :param str local_file_name: file name of zip file
    :return: list of archive members by name.
    """
    local_file_name = op.expanduser(local_file_name)
    destination = op.dirname(local_file_name)
    datafile = zipfile.ZipFile(local_file_name)
    namelist = datafile.namelist()
    datafile.extractall(destination)
    datafile.close()
    remove(local_file_name)

    fullnamelist = []
    for fn in namelist:
        fullnamelist.append(op.join(destination, fn))
    return fullnamelist


def unzip_recursive(zip_file_name):
    """Unzip file with all recursive zip files inside and delete zip files after that.

    :param zip_file_name: file name of zip file
    :return: list of archive members by name.
    """
    logger.debug("unzipping " + zip_file_name)
    fnlist = unzip_one(zip_file_name)
    for fn in fnlist:
        if zipfile.is_zipfile(fn):
            local_fnlist = unzip_recursive(fn)
            fnlist.extend(local_fnlist)
    return fnlist


class ExtendAction(argparse.Action):

    def __call__(self, parser, namespace, values, option_string=None):
        items = getattr(namespace, self.dest) or []
        items.extend(values)
        setattr(namespace, self.dest, items)


def main(turn_on_logging=False):
    if turn_on_logging:
        # logger = logging.getLogger()
        # logger.setLevel(logging.DEBUG)
        main_logger = logging.getLogger(__file__ + "datasets commandline")

        main_logger.setLevel(logging.WARNING)
        ch = logging.StreamHandler()
        main_logger.addHandler(ch)
    else:
        logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.DEBUG)
        main_logger = logger

    # logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(
        description="Work on dataset")
    parser.register('action', 'extend', ExtendAction)
    parser.add_argument(
        "-l", "--labels", metavar="N", nargs="+", action="extend",
        default=None,
        help='Get sample data')
    parser.add_argument(
        '-L', '--print_labels', action="store_true",
        default=False,
        help='print all available labels')
    parser.add_argument(
        '-c', '--checksum',  # action="store_true",
        default=None,
        help='Get hash for requested path')
    parser.add_argument(
        '-v', '--verbose', action="store_true",
        default=False,
        help='more messages')
    parser.add_argument(
        '-d', '--debug',  # action="store_true",
        default=None,
        help='Set debug level')
    parser.add_argument(
        '-o', '--destination_dir',
        default=op.join(dataset_path(get_root=True), "medical", "orig"),
        help='Set output directory. If not used, the standard dataset dir is used')
    parser.add_argument(
        '-sdp', '--set_dataset_path',
        default=None,
        help='Set standard dataset path')
    parser.add_argument(
        '-gdp', '--get_dataset_path',
        default=False, action="store_true",
        help='Get standard dataset path')
    parser.add_argument(
        '--dry_run', action="store_true",
        default=False,
        help='Do not download')

    args = parser.parse_args()

    # if args.get_sample_data == False and args.install == False and args.build_gco == False:
    # default setup is install and get sample data
    #        args.get_sample_data = True
    #        args.install = True
    #        args.build_gco = False
    if args.verbose:
        # logger.setLevel(logging.DEBUG)
        main_logger.setLevel(logging.INFO)
    if args.debug is not None:
        main_logger.setLevel(int(args.debug))

    if args.set_dataset_path is not None:
        set_dataset_path(args.set_dataset_path)
        logger.info("Dataset path changed")
        return

    if args.get_dataset_path:
        # dp = dataset_path()
        dp = op.join(dataset_path(get_root=True), "medical", "orig"),
        logger.info(dp)
        print(dp)
        # logger.info("Dataset path changed")
        return

    if args.checksum is not None:
        print(checksum(args.checksum))
        if args.labels is None:
            return
    if args.print_labels:
        print(sorted(data_urls.keys()))
        return

    if args.labels is not None:
        main_logger.info("Downloading labels: {}".format(args.labels))
        download(args.labels, destination_dir=args.destination_dir, dry_run=args.dry_run)

    # submodule_update()


if __name__ == "__main__":
    main(turn_on_logging=False)



#! /usr/bin/env python
# -*- coding: utf-8 -*-


import logging
logger = logging.getLogger(__name__)

import numpy as np

def get_pixel_array_from_sitk(sitk_image):
    dim = sitk_image

    import SimpleITK as sitk
    if dim.HasMetaDataKey("0028|1052") and dim.HasMetaDataKey("0028|1053"):
        rescale_intercept = dim.GetMetaData("0028|1052")
        rescale_slope = dim.GetMetaData("0028|1053")
        slope, inter = get_slope_and_intercept_from_strings(rescale_slope, rescale_intercept)
    else:
        slope = 1
        inter = 0

    data3d = sitk.GetArrayFromImage(sitk_image)  # + 1024
    data3d = rescale_pixel_array(data3d, slope=slope, inter=inter)

    return data3d

def get_sitk_image_from_ndarray(data3d):
    """
    Prepare SimpleItk Image object and rescale data to unsigned types.

    Simple ITK with version higher than 1.0.0 can not write signed int16. This function check
    the SimpleITK version and use work around with Rescale Intercept and Rescale Slope
    :param data3d:
    :return:
    """

    import SimpleITK as sitk
    rescale_intercept = None
    if sitk.Version.MajorVersion() > 0:
        if data3d.dtype == np.int8:
            rescale_intercept = -2**7
            data3d = (data3d - rescale_intercept).astype(np.uint8)
        elif data3d.dtype == np.int16:
            # simpleitk is not able to store this. It uses only 11 bites
            # rescale_intercept = -2**15
            rescale_intercept = -2**10
            data3d = (data3d - rescale_intercept).astype(np.uint16)
        elif data3d.dtype == np.int32:
            rescale_intercept = -2**31
            data3d = (data3d - rescale_intercept).astype(np.uint16)

    dim = sitk.GetImageFromArray(data3d)
    if sitk.Version.MajorVersion() > 0:
        if rescale_intercept is not None:
            # rescale slope (0028|1053), rescale intercept (0028|1052)
            dim.SetMetaData("0028|1052", str(rescale_intercept))
            dim.SetMetaData("0028|1053", "1")

    return dim

def get_slope_and_intercept_from_strings(rescale_slope, rescale_intercept):
    if type(rescale_slope) is str:
        slope = float(rescale_slope)
    else:
        slope = rescale_slope

    if type(rescale_intercept) is str:
        inter = float(rescale_intercept)
    else:
        inter = rescale_intercept

    return slope, inter

def get_slope_and_intercept_from_pdcm(dcmdata):
    """
    Get scale and intercept from pydicom file object.
    :param dcmdata:
    :return:
    """

    if hasattr(dcmdata, "RescaleSlope") and hasattr(dcmdata, "RescaleIntercept"):
        rescale_slope = dcmdata.RescaleSlope
        rescale_intercept = dcmdata.RescaleIntercept
        slope, inter = get_slope_and_intercept_from_strings(rescale_slope, rescale_intercept)
    else:
        slope = 1
        inter = 0

    return slope, inter

# def get_pixel_array_from_pdcm(data):
#     """
#     Get data2d and rescale
#     :param data: pydicom dcmobj
#     :return: data2d, original_data2d_dtype
#     """
#     from . import dcmtools
#     data2d = data.pixel_array
#     slope, inter = get_slope_and_intercept_from_pdcm(data)
#     # new_data2d = rescale_pixel_array(data2d, slope, inter)
#     return data2d, slope, inter

from .misc import use_economic_dtype as rescale_pixel_array

# def rescale_pixel_array(data2d, slope, inter, dtype=None):
#     from .misc import
#     orig_dtype = data2d.dtype
#     if dtype is None:
#         if orig_dtype is np.uint16 and inter == -2**15:
#             dtype = np.int16
#         elif orig_dtype is np.uint16 and inter < 0:
#             dtype = np.int16
#         elif orig_dtype is np.uint32 and inter < 0:
#             dtype = np.int32
#         else:
#             dtype = orig_dtype
#
#     new_data2d = ((np.float(slope) * data2d) + np.float(inter)).astype(dtype)
#     return new_data2d



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# import sys
import os

import logging
logger = logging.getLogger(__name__)

import sys
import os.path
import tarfile
import numpy as np

path_to_script = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(path_to_script, "./extern/sPickle"))


def get_tar_output_dir(fname):
    if (fname.endswith("tar.bz2")):
        fname = fname[:-8]
        return fname

def untar(fname, path=".", output_dir_by_filename=True, force_rewrite=False):
    if output_dir_by_filename:
        path= get_tar_output_dir(fname)
    if os.path.exists(path) and force_rewrite is False:
        logger.debug("Extraction of tar file interrupted. Directory %s exists" % path)
        return path

    # extraction
    if (fname.endswith("tar.bz2")):
        if path != ".":
            os.makedirs(path)
        tar = tarfile.open(fname)

        tar.extractall(path=path)
        tar.close()
        logger.debug("Extracted in " + str(path))
        return path
    else:
        logger.warning("Not a tar.bz2 file: '%s '" % path)





#! /usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
# import funkcí z jiného adresáře
import unittest
import os
import os.path as op
import warnings

# path_to_script = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src/"))
# sys.path.append(os.path.join(path_to_script, "../extern/py3DSeedEditor/"))
# sys.path.append(os.path.join(path_to_script, "../src/"))

import shutil


import numpy as np

from nose.plugins.attrib import attr

try:
    import dicom
    dicom.debug(False)
except:
    import pydicom as dicom
    dicom.config.debug(False)

#
import io3d
import io3d.datawriter as dwriter
import io3d.datareader as dreader

# import sed3 as pyed
SAMPLE_DATA_DIR = io3d.datasets.join_path("sample_data")
# SAMPLE_DATA_DIR = "~/data/medical/orig/sample_data"



class DicomWriterTest(unittest.TestCase):
    # def setUp(self):

    #     import imtools
    #     import imtools.sample_data
    #     imtools.sample_data.get_sample_data(["jatra_5mm", "volumetrie"], SAMPLE_DATA_DIR)

    # def test_write_h5(self):
    #     """
    #     Reads dicom data and from slices and stores into one Dicom file. This
    #     file is then readed and compared with input data.
    #     """
    #     filename = 'tests/output_dcm3d.dcm'
    #     dr = dreader.DataReader()
    #     data3d, metadata = dr.Get3DData(
    #         'sample_data/jatra_5mm/'
    #         # 'sample_data/volumetrie/'
    #     )
    #     # import pdb; pdb.set_trace()
    #
    #
    #     dw = dwriter.DataWriter()
    #     dw.Write3DData(data3d, filename, filetype='dcm', metadata=metadata)
    #
    #     data3d_n, metadata_n = dr.Get3DData(filename)
    #     self.assertEqual(data3d[10, 11, 12], data3d_n[10, 11, 12])
    #     os.remove(filename)

    @attr('dataset')
    def test_write_dicom_from_scratch(self):
        """
        Reads dicom data and from slices and stores into one Dicom file. This
        file is then readed and compared with input data.
        """
        filename = 'tests/output_dcm3d.dcm'
        dr = dreader.DataReader()
        path_to_data = op.join(SAMPLE_DATA_DIR, 'jatra_5mm')
        logger.debug(SAMPLE_DATA_DIR)
        logger.debug(path_to_data)
        data3d, metadata = dr.Get3DData(
            path_to_data, dataplus_format=False
            # 'sample_data/volumetrie/'
        )
        dw = dwriter.DataWriter()
        dw.Write3DData(data3d, filename, filetype='dcm', metadata=metadata)

        data3d_n, metadata_n = dr.Get3DData(filename, dataplus_format=False)
        self.assertEqual(data3d[10, 11, 12], data3d_n[10, 11, 12])
        os.remove(filename)

#    def setUp(self):
#        self.dcmdir = os.path.join(path_to_script, '../sample_data/jatra_5mm')
# self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
#        reader = dcmr.DicomReader(self.dcmdir)
#        self.data3d = reader.get_3Ddata()
#        self.metadata = reader.get_metaData()
    def test_write_and_read(self):
        filename = 'test_file.dcm'
        data = (np.random.random([30, 100, 120]) * 30).astype(np.int16)
        data[0:5, 20:60, 60:70] += 30
        metadata = {'voxelsize_mm': [1, 2, 3]}
        dw = dwriter.DataWriter()
        logger.debug(filename)
        dw.Write3DData(data, filename, filetype='dcm', metadata=metadata)

        dr = dreader.DataReader()
        newdata, newmetadata = dr.Get3DData(filename, dataplus_format=False)

        # print("meta ", metadata)
        # print("new meta ", newmetadata)

        # hack with -1024, because of wrong data reading
        self.assertEqual(data[10, 10, 10], newdata[10, 10, 10])
        self.assertEqual(data[2, 10, 1], newdata[2, 10, 1])
        # print(metadata["voxelsize_mm"])
        # print(newmetadata["voxelsize_mm"])
        self.assertEqual(metadata['voxelsize_mm'][0],
                         newmetadata['voxelsize_mm'][0])
        self. assertEqual(metadata['voxelsize_mm'][1],
                          newmetadata['voxelsize_mm'][1])
        self.assertEqual(metadata['voxelsize_mm'][2],
                         newmetadata['voxelsize_mm'][2])
        os.remove(filename)

    def test_write_and_read_pklz(self):
        filename = 'test_file.pklz'
        data = (np.random.random([30, 100, 120]) * 30).astype(np.int16)
        data[0:5, 20:60, 60:70] += 30
        metadata = {'voxelsize_mm': [1, 2, 3]}
        dw = dwriter.DataWriter()
        dw.Write3DData(data, filename, filetype='auto', metadata=metadata)

        dr = dreader.DataReader()
        newdata, newmetadata = dr.Get3DData(filename, dataplus_format=False)

        # print("meta ", metadata)
        # print("new meta ", newmetadata)

        # hack with -1024, because of wrong data reading
        self.assertEqual(data[10, 10, 10], newdata[10, 10, 10])
        self.assertEqual(data[2, 10, 1], newdata[2, 10, 1])
        self.assertEqual(metadata['voxelsize_mm'][0],
                         newmetadata['voxelsize_mm'][0])


    @unittest.skip('SliceThickness is not stored in dicom files')
    def test_write_dcm_slices_datap(self):
        # Z nějakého důvodu plošné snímky neukládají tloušťku řezu. Jinak by test měl fungovat.
        datadir = "test_dcm"
        filename = datadir + '/test_file{slice_position:07.3f}.dcm'
        data = (np.random.random([30, 100, 120]) * 30).astype(np.int16)
        data[0:5, 20:60, 60:70] += 30
        datap = {'voxelsize_mm': [1, 2, 3], "data3d": data}
        dw = dwriter.DataWriter()
        dw.Write3DData(datap, filename, filetype='auto')

        dr = dreader.DataReader()
        newdata, newmetadata = dr.Get3DData(datadir, dataplus_format=False)

        # print("meta ", metadata)
        # print("new meta ", newmetadata)

        # hack with -1024, because of wrong data reading
        self.assertEqual(data[10, 10, 10], newdata[10, 10, 10])
        self.assertEqual(data[2, 10, 1], newdata[2, 10, 1])
        self.assertEqual(datap['voxelsize_mm'][0],
                         newmetadata['voxelsize_mm'][0])
        # os.removedirs()
        shutil.rmtree(datadir)

    @attr("slow")
    def test_read_mhd_and_write_pklz(self):
        """
        test data on sliver dataset
        :return:
        """
        # infn = op.join(op.expanduser(io3d.datasets.local_dir), "sliver07/training/liver-orig001.mhd")
        infn = io3d.datasets.join_path("liver-orig001.mhd")
        datap = io3d.read(infn, dataplus_format=True)
        datap["segmentation"] = np.zeros_like(datap["data3d"], dtype=np.uint8)

        io3d.write(datap, path="test_mhd.pklz")

    def test_write_hdf5(self):
        filename = 'test_file.hdf5'
        data = (np.random.random([30, 100, 120]) * 30).astype(np.int16)
        data[0:5, 20:60, 60:70] += 30
        metadata = {'voxelsize_mm': [1, 2, 3]}
        dwriter.write(data, filename, filetype='hdf5', metadata=metadata)


    # @attr('interactive')
    # @attr('interactive')
    def test_write_and_read_hdf5(self):
        filename = 'test_file.hdf5'
        data = (np.random.random([30, 100, 120]) * 30).astype(np.int16)
        data[0:5, 20:60, 60:70] += 30
        metadata = {'voxelsize_mm': [1, 2, 3]}
        metadata[5] = "not a number"
        metadata[7] = ["not a number"]
        dwriter.write(data, filename, filetype='hdf5', metadata=metadata)

        newdata, newmetadata = dreader.read(filename, dataplus_format=False)

        # hack with -1024, because of wrong data reading
        self.assertEqual(data[10, 10, 10], newdata[10, 10, 10])
        self.assertEqual(data[2, 10, 1], newdata[2, 10, 1])
        self.assertEqual(newmetadata['voxelsize_mm'][0],
                         newmetadata['voxelsize_mm'][0])
        self.assertEqual(newmetadata[7][0],
                         newmetadata[7][0])
# @TODO there is a bug in SimpleITK. slice voxel size must be same
        # self. assertEqual(metadata['voxelsize_mm'][1],
        #                   newmetadata['voxelsize_mm'][1])
        self.assertEqual(metadata['voxelsize_mm'][2],
                         newmetadata['voxelsize_mm'][2])
        os.remove(filename)

    def test_write_single_dicom(self):
        filename = 'tests_outputs/single_test_file.dcm'
        filedir = os.path.dirname(filename)
        data3d = np.zeros([10,10,10], dtype=np.uint8)
        data3d [1:, 5, 3:9] = 124
        data3d [3:, 2:7, 4] = 60

        io3d.write(data3d, filename)
        os.remove(filename)

    def test_add_overlay_and_read_one_file_with_overlay(self):
        logger.info("test started")
        logger.info(logger.handlers)
        filename = 'tests_outputs/test_file.dcm'
        filedir = os.path.dirname(filename)

        # number of tested overlay
        i_overlay = 6

        if not os.path.exists('tests_outputs'):
            os.mkdir('tests_outputs')

        data = (np.random.random([30, 100, 120]) * 30).astype(np.int16)
        data[0:5, 20:60, 60:70] += 30
        overlay = np.zeros([512, 512], dtype=np.uint8)
        overlay[450:500, 30:100] = 1

        # metadata = {'voxelsize_mm': [1, 2, 3]}
        dw = dwriter.DataWriter()
        dw.add_overlay_to_slice_file(
            # 'sample_data/jatra_5mm/IM-0001-0019.dcm',
            op.join(SAMPLE_DATA_DIR, 'volumetrie/volumetry_slice.DCM'),
            overlay,
            i_overlay,
            filename
        )
        logger.info("data saved")
        # import ipdb; ipdb.set_trace()
        # print("asdfa")
        dr = dreader.DataReader()
        newdata, newmetadata = dr.Get3DData('tests_outputs', dataplus_format=False)
        newoverlay = dr.get_overlay()
        # print(newoverlay)

        # ed = pyed.py3DSeedEditor(newoverlay[6])
        # ed.show()
        # print("oioiu")
        self.assertTrue((newoverlay[i_overlay] == overlay).all())

        # os.remove(filename)
        shutil.rmtree(filedir)

    def test_add_overlay_to_copied_dir(self):
        """
        writes 3d label to copied dicom files
        """
        filedir = 'test_outputs_dir'
        n_files = 3

        # number of tested overlay
        i_overlay = 6

        if not os.path.exists(filedir):
            os.mkdir(filedir)

# open copied data to obtain dcmfilefilelist
        dr = dreader.DataReader()
        data3d, metadata = dr.Get3DData(
            op.join(SAMPLE_DATA_DIR, 'jatra_5mm/'), dataplus_format=False
            # 'sample_data/volumetrie/'
        )
# for test we are working only with small number of files (n_files)
        metadata['dcmfilelist'] = metadata['dcmfilelist'][:n_files]

# create overlay
        overlay = np.zeros([n_files, 512, 512], dtype=np.uint8)
        overlay[:, 450:500, 30:100] = 1
# if there is more slides, try more complicated overlay
        overlay[0, 430:460, 20:110] = 1
        overlay[-1:, 470:520, 10:120] = 1

        overlays = {i_overlay: overlay}

        dw = dwriter.DataWriter()
        dw.DataCopyWithOverlay(metadata['dcmfilelist'], filedir, overlays)

# try read written data
        dr = dreader.DataReader()
        newdata, newmetadata = dr.Get3DData(filedir, dataplus_format=False)

        # test old function

        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            newoverlay = dr.GetOverlay()

        newoverlay = dr.get_overlay()


        self.assertTrue((newoverlay[i_overlay] == overlays[i_overlay]).all())

        # os.remove(filename)
        shutil.rmtree(filedir)

    def generate_waving_data(self, szx, szy, szz, value=150, dtype=np.uint8):
        """
        generating smooth non constant data
        """
        data3d = np.zeros([szz, szx, szx], dtype=dtype)
        for i in range(0, data3d.shape[0]):
            x = int(np.sin(i*2*np.pi/(szz - 40.0))*((szx-2)/2) + szx/2)
            y = int(np.cos(i*2*np.pi/(0.3*(szz - 4.0)))*((szy-2)/2) + szy/2)
            # x = int(np.sin(i*2*np.pi/40.0)*((szx-2)/2) + szx/2)
            # print(x, '   ', y)
            data3d[i, 0:x, y:-1] = value
        return data3d

    @attr('actual')
    def test_save_image_stack_based_on_filename(self):
        testdatadir = 'test_svimstack2'
        if os.path.exists(testdatadir):
            shutil.rmtree(testdatadir, ignore_errors=True)
        szx = 30
        szy = 20
        szz = 120
        data3d = self.generate_waving_data(
            szx, szy, szz, value=150, dtype=np.uint8)
        # import sed3
        # ed = sed3.sed3(data3d)
        # ed.show()
        #
        io3d.write(data3d, testdatadir + "/soubory{:04d}.tiff")



        dr = dreader.DataReader()
        data3dnew, metadata = dr.Get3DData(
            testdatadir, dataplus_format=False
            # 'sample_data/volumetrie/'
        )
        # import sed3
        # ed = sed3.sed3(data3dnew)
        # ed.show()
        self.assertEqual(
            np.sum(np.abs(data3d - data3dnew)),
            0
        )
        shutil.rmtree(testdatadir)

    def test_save_image_stack_with_unique_series_number_based_on_filename(self):
        testdatadir = 'test_svimstack2'
        if os.path.exists(testdatadir):
            shutil.rmtree(testdatadir, ignore_errors=True)
        szx = 30
        szy = 20
        szz = 120
        data3d = self.generate_waving_data(
            szx, szy, szz, value=150, dtype=np.uint8)
        io3d.write(data3d, testdatadir + "/{series_number:03d}/soubory{slice_position:07.3f}.tiff")
        # second time there should be directory 002/
        io3d.write(data3d, testdatadir + "/{series_number:03d}/soubory{slice_position:07.3f}.tiff")



        dr = dreader.DataReader()
        data3dnew, metadata = dr.Get3DData(
            testdatadir + "/002/", dataplus_format=False
            # 'sample_data/volumetrie/'
        )
        # import sed3
        # ed = sed3.sed3(data3dnew)
        # ed.show()
        self.assertEqual(
            np.sum(np.abs(data3d - data3dnew)),
            0
        )
        shutil.rmtree(testdatadir)

    def test_save_image_stack(self):
        testdatadir = 'test_svimstack'
        if os.path.exists(testdatadir):
            shutil.rmtree(testdatadir, ignore_errors=True)
        szx = 30
        szy = 20
        szz = 120
        data3d = self.generate_waving_data(
            szx, szy, szz, value=150, dtype=np.uint8)
        # import sed3
        # ed = sed3.sed3(data3d)
        # ed.show()
        #
        dw = dwriter.DataWriter()

        dw.save_image_stack(data3d, testdatadir + '/soubory.png')
        dr = dreader.DataReader()
        data3dnew, metadata = dr.Get3DData(
            testdatadir, dataplus_format=False
            # 'sample_data/volumetrie/'
        )
        # import sed3
        # ed = sed3.sed3(data3dnew)
        # ed.show()
        self.assertEqual(
            np.sum(np.abs(data3d - data3dnew)),
            0
        )
        shutil.rmtree(testdatadir)

    def test_save_image_stack_with_writer(self):
        testdatadir = 'test_svimstack_with_writer'
        data3d = self.generate_waving_data(
            szx=30, szy=20, szz=10, value=150, dtype=np.uint8)
        dw = dwriter.DataWriter()

        dw.Write3DData(data3d, testdatadir + '/soubory.png',
                       filetype='image_stack')
        dr = dreader.DataReader()
        data3dnew, metadata = dr.Get3DData(
            testdatadir, dataplus_format=False
        )
        # import sed3
        # ed = sed3.sed3(data3dnew)
        # ed.show()
        self.assertEqual(
            np.sum(np.abs(data3d - data3dnew)),
            0
        )
        shutil.rmtree(testdatadir)

    def test_SimpleITK(self):
        logger.debug(logger)
        logger.debug("test SimpleITK started")
        path = "new_dcmfile.dcm"
        data3d = np.zeros([10, 15, 20], dtype='uint16')
        vsz = np.asarray([2,3,1.5])

        import SimpleITK as sitk
        dim = sitk.GetImageFromArray(data3d)
        dim.SetSpacing([vsz[1], vsz[2], vsz[0]])
        sitk.WriteImage(dim, path)


    def test_fill_series_number(self):
        import io3d.datawriter

        out = io3d.datawriter.filepattern_fill_series_number("{seriesn:03d}/{slicen:06d}", series_number=15)
        self.assertEqual(out, '015/{slicen:06d}')

    @attr('interactive')
    def test_read_data_without_slice_thickness(self):
        """ data without SliceThickness
        :return:
        """

        dr = dreader.DataReader()
        data3dnew, metadata = dr.Get3DData(
                "~/data/medical/orig/dicom_test_claudio", dataplus_format=False
        )
        metadata


if __name__ == "__main__":
    unittest.main()



import logging
logger = logging.getLogger(__name__)
import glob
try:
    import pydicom
except ImportError:
    import warnings
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        import dicom as pydicom
    logger.debug("dicom imported - it would be better use pydicom")

class Anonymizer():
    def __init__(self):
        self.filename = ''
        self.output_filename = ''
        self.whitelist = None

    def file_anonymization(self, filename, output_filename=None):
        self.filename = filename
        self.output_filename = output_filename
        if(self.output_filename == None):
            self.output_filename = 'D:\jatra_test'
        if(self.whitelist == None):
            self.setWhitelistDefault()

        dcmobj = pydicom.read_file(self.filename)
        atrr_list = dir(dcmobj)
        check = None

        for atribut in atrr_list:
            for item in self.whitelist:
                if(atribut != item):
                    check = False
                if(atribut == item):
                    check = True
        if(check == False):
            setattr(dcmobj, atribut, "")
        check = None

        dcmobj.save_as(self.output_filename)

    def recursive_anonymization(self, path, output_path=None):
        dirlist = glob.glob(path)

        pass

     ## gives path to whitelist text (.txt) file
    def setWhitelist(self, whitelistPath):
        self.whitelistPath = whitelistPath
        pthR = whitelistPath
        handleOpen = open(pthR, "r")

        with open(pthR) as file_handler:
            for line in file_handler:
                self.whitelist.append(line)
        handleOpen.close()

    def setWhitelistDefault(self):
        self.whitelist = [
            'Modality',
            'PatientAge',
            'AcquisitionDate',
            'AcquisitionNumber',
            'AcquisitionTime',
            'PatientPosition',
            'PatientSex',
            'RescaleSlope',
            'RescaleIntercept',
            'PixelSpacing',
            'SeriesDate',
            'SeriesDescription',
            'SeriesTime',
            'SeriesNumber',
            'SliceLocation',
            'SliceThickness',
            'StudyDate',
            'StudyID',
            'StudyTime',
            'StudyDesciption',
            'OverlayType',
            'OverlayRows',
            'OverlayOrigin',
            'OverlayDescription',
            'OverlayData',
            'OverlayColumns',
            'OverlayBitsAllocated',
            'OverlayBitPosition',
            'ImageComments',
            'ImageFrameOrigin',
            'ImageOrientationPatient',
            'ImagePositionPatient',
            ]




#! /usr/bin/env python
# -*- coding: utf-8 -*-


import logging
logger = logging.getLogger(__name__)
# import funkcí z jiného adresáře
# import sys
import os.path
import os.path as op
# import copy

import unittest
from nose.plugins.attrib import attr
# sample_data_path = os.path.dirname(os.path.abspath(__file__))
# sample_data_path
# sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src/"))
# sys.path.append(os.path.join(path_to_script, "../extern/py3DSeedEditor/"))
# sys.path.append(os.path.join(path_to_script, "../src/"))

from PyQt4.QtGui import QFileDialog, QApplication, QMainWindow
import sys

import numpy as np

try:
    import dicom as pydicom
    pydicom.debug(False)
except:
    import pydicom
    pydicom.config.debug(False)

from distutils.version import LooseVersion
#
import io3d
import io3d.dcmreaddata as dcmr
import io3d.fsbrowser
sample_data_path = "~/data/medical/orig/sample_data/"
sample_data_path = op.expanduser(sample_data_path)

class FileSystemBrowserTest(unittest.TestCase):

    # @unittest.skip('waiting for implementation')
    @unittest.skipIf(LooseVersion(pydicom.__version__) < LooseVersion("1.0.0"), "Data from pydicom >= 1.0.0 required")
    def test_fsbrowser_path_info(self):
        import pydicom.data

        # filepath = pydicom.data.get_testdata_files('DICOMDIR')[0]
        filepath = op.join(pydicom.data.DATA_ROOT, 'test_files/dicomdirtests/98892001/CT2N')
        fsb = io3d.fsbrowser.FileSystemBrowser(filepath)
        dirlist = fsb.get_path_info(filepath)
        self.assertTrue("path" in dirlist)
        self.assertTrue("name" in dirlist)

    #  comment next line if you want to run the test
    # @unittest.skip('waiting for implementation')
    @unittest.skipIf(LooseVersion(pydicom.__version__) < LooseVersion("1.0.0"), "Data from pydicom >= 1.0.0 required")
    def test_fsbrowser_dir_list(self):

        import pydicom.data
        # TODO make test stronger
        # filepath = pydicom.data.get_testdata_files('DICOMDIR')[0]
        filepath = op.join(pydicom.data.DATA_ROOT, 'test_files/dicomdirtests/98892001/CT2N')
        # filepath = io3d.datasets.join_path("3Dircadb1.1/PATIENT_DICOM")
        # filepath = io3d.datasets.join_path("jatra_5mm")
        fsb = io3d.fsbrowser.FileSystemBrowser(filepath)
        dirlist = fsb.get_dir_list()
        self.assertTrue("path" in dirlist[0])
        self.assertTrue("name" in dirlist[0])

    @unittest.skip('technology test')
    def test_devel_qt_dialog_fsbrowser(self):
        import PyQt4
        import sys
        from PyQt4.QtGui import QApplication, QWidget, QLineEdit, QPushButton, QImage, QPixmap, QLabel
        import matplotlib.pyplot as plt

        # Create an PyQT4 application object.
        a = QApplication(sys.argv)

        # The QWidget widget is the base class of all user interface objects in PyQt4.
        w = QWidget()

        # Set window size.
        w.resize(320, 240)

        # Set window title
        w.setWindowTitle("Hello World!")

        # Get filename using QFileDialog
        # filename = QFileDialog.getOpenFileName(w, 'Open File', '/')
        # print(filename)

        qfd = QFileDialog(None)

        lineedit = QLineEdit(qfd)
        # Create a button in the window
        button = QPushButton('Click me', qfd)


        # Image
        image = io3d.datasets.generate_face([1, 100, 100]).squeeze()

        cmap = np.uint8(np.round(255 * plt.get_cmap('magma')(np.arange(256))))
        image /= image.max()
        image = np.minimum(image, 1.0)
        image = np.round(255 * image).astype('uint8')
        Y, X = image.shape
        self._bgra = np.zeros((Y, X, 4), dtype=np.uint8, order='C')
        self._bgra[..., 0] = cmap[:, 2][image]
        self._bgra[..., 1] = cmap[:, 1][image]
        self._bgra[..., 2] = cmap[:, 0][image]
        qimage = QImage(self._bgra.data, X, Y, QImage.Format_RGB32)

        pixmap = QPixmap.fromImage(qimage)

        label = QLabel(w)
        # pixmap = QPixmap(os.getcwd() + 'https://pythonspot-9329.kxcdn.com/logo.png')
        label.setPixmap(pixmap)
        w.resize(pixmap.width(), pixmap.height())
        # success = img.loadFromData(image_data)
        # painter = QPainter(img)
        layout = qfd.layout()

        layout.addWidget(lineedit)
        layout.addWidget(button, 2, 5)
        layout.addWidget(label, 1, 5)

        if qfd.exec_():
            print(qfd.selectedFiles())
        else:
            print("Cancel")

        # a.exec_()

    def test_remove(self):
        fn = "try_to_remove.txt"
        io3d.remove_if_exists(fn)
        with open(fn, "w") as f:
            f.write("ahoj")

        io3d.remove_if_exists(fn)

        self.assertFalse(op.exists(fn))


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8

import logging
logger = logging.getLogger(__name__)

import unittest
from io3d import dili


class DictListTestCase(unittest.TestCase):
    def test_ditc_flatten(self):
        data = {
            'a': 1,
            'b': 2,
            'c': {
                'aa': 11,
                'bb': 22,
                'cc': {
                    'aaa': 111
                }
            }
        }
        dct = dili.flatten_dict(data)
        dct = dict(dct)
        self.assertIn("cccaaa", dct.keys())

    def test_ditc_flatten_with_separator(self):
        data = {
            'a': 1,
            'b': 2,
            'c': {
                'aa': 11,
                'bb': 22,
                'cc': {
                    'aaa': 111
                }
            }
        }
        dct = dili.flatten_dict(data, separator=";")
        dct = dict(dct)
        self.assertIn("c;cc;aaa", dct.keys())

    def test_list_filter(self):
        lst = ["aa", "sss", "aaron", "rew"]
        output = dili.list_filter(lst, notstartswith="aa")
        self.assertTrue(["sss", "rew"] == output)

    def test_dict_find_key(self):
        slab={"liver": 1, "porta": 2}
        val = dili.dict_find_key(slab, 2)
        self.assertEqual(val, "porta")

    # def test_dict_find_key_error(self):
    #     slab={"liver": 1, "porta": 2}
    #     val = dili.dict_find_key(slab, 3)
    #     self.assertEqual(val, "porta")
    def sort_list_data(self):
        dct = [
            {"name": "mira", "age": 34, "height": 172.0, "weight": 75},
            {"name": "kamca", "age": 25, "height": 152.0, "weight": 55},
            {"name": "bob", "age": 34, "height": 183.0, "weight": 85},
            {"name": "pavel", "age": 34, "height": 179.0, "weight": 98},
            {"name": "veru", "age": 25, "height": 162.0, "weight": 60},
            {"name": "pepa", "age": 39, "height": 182.0, "weight": 130},
        ]
        return dct

    def test_sort_list_of_dicts(self):
        dct = self.sort_list_data()

        dct = dili.sort_list_of_dicts(dct, keys=["age", "height"])
        self.assertEqual(dct[0]["name"], "kamca")
        self.assertEqual(dct[1]["name"], "veru")
        self.assertEqual(dct[2]["name"], "mira")
        self.assertEqual(dct[-1]["name"], "pepa")

    def test_sort_list_of_dicts_single_key(self):
        dct = self.sort_list_data()
        dct = dili.sort_list_of_dicts(dct, keys="height")
        self.assertEqual(dct[0]["name"], "kamca")
        # self.assertEqual(dct[1]["name"], "veru")
        # self.assertEqual(dct[2]["name"], "mira")
        self.assertEqual(dct[-1]["name"], "bob")
def main():
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2015 mjirik <mjirik@mjirik-Latitude-E6520>
#
# Distributed under terms of the MIT license.

"""
Module for testing format rawiv
"""
import logging
logger = logging.getLogger(__name__)

import unittest
import os.path as op
import shutil
import sys
import numpy as np
import io3d

from nose.plugins.attrib import attr

class DatasetsTest(unittest.TestCase):

    # @attr('actual')
    # @attr('interactive')
    def test_get_path(self):
        path = op.join(io3d.datasets.dataset_path(get_root=True), "medical", "orig")
        self.assertEqual(type(path), str)

    def test_download_package_dry_run(self):
        dataset_labels = ["lisa"]
        new_dataset_labels = io3d.datasets._expand_dataset_packages(dataset_labels)
        self.assertGreater(len(new_dataset_labels), 1)

    def test_download_package_dry_run(self):
        logger.debug("download in progress")
        io3d.datasets.download("lisa", dry_run=True)

    def test_download(self):

        io3d.datasets.download("gensei_slices")
        pth = io3d.datasets.join_path("gensei_slices")
        logger.debug(pth)
        self.assertTrue(op.exists(pth))
        # import sed3
        # ed = sed3.sed3(data3d)
        # ed.show()
        # on this index sould be number 119
        # self.assertEqual(data3d[29, 13, 27], 119)
        pass

    def test_download_to_dir(self):
        pth = op.join("./tmp/", "biodur_sample")
        if op.exists(pth):
            shutil.rmtree(pth)

        io3d.datasets.download("biodur_sample", "./tmp/")
        pth = op.join("./tmp/", "biodur_sample")
        self.assertTrue(op.exists(pth))

    def test_change_dataset_path(self):
        dp_new1 = "~/io3d_test1_dataset_dir/"
        dp_new2 = "~/io3d_test2_dataset_dir/"
        dp_old = io3d.datasets.join_path(get_root=True)

        # change once
        io3d.datasets.set_dataset_path(dp_new1)
        dp_joined1 = io3d.datasets.join_path("jatra_5mm")
        self.assertTrue(dp_joined1.find("io3d_test1_dataset_dir") > 0)

        # second change
        io3d.datasets.set_dataset_path(dp_new2)
        dp_joined2 = io3d.datasets.join_path("jatra_5mm")
        self.assertTrue(dp_joined2.find("io3d_test2_dataset_dir") > 0)

        # return path back
        io3d.datasets.set_dataset_path(dp_old)

    @attr('slow')
    def test_getold(self):
        io3d.datasets.download("3Dircadb1.1")
        io3d.datasets.get_old("3Dircadb1", "*1/P*")

    @unittest.skip("waiting for implementation of get() function")
    def test_get_no_series_number(self):

        datap = io3d.datasets.get("jatra_5mm")
        self.assertEqual(datap["data3d"].shape[1], 512)


    @unittest.skip("waiting for implementation of get() function")
    def test_get_with_series_number(self):
        # remove data if they are stored
        shutil.rmtree(io3d.daatsets.join("path_where_the_data_are_usualy_stored"))

        # first read
        datap = io3d.datasets.get("3Dircadb1", 3)
        self.assertEqual(datap["data3d"].shape[1], 512)

        # second read should be faster
        datap = io3d.datasets.get("3Dircadb1", 3)
        self.assertEqual(datap["data3d"].shape[1], 512)

    def test_main_list_labels(self):
        tmpargv = sys.argv
        sys.argv = [tmpargv[0], "-L"]
        io3d.datasets.main()
        sys.argv = tmpargv

    def test_main_get_dataset_path(self):

        tmpargv = sys.argv
        sys.argv = [tmpargv[0], "-gdp"]
        io3d.datasets.main()
        sys.argv = tmpargv

    def test_main_get_dataset_multiple_labels(self):

        tmpargv = sys.argv
        sys.argv = [tmpargv[0], "-l", "jatra_5mm", "head", "-l", "exp", "--dry_run"]
        io3d.datasets.main()
        sys.argv = tmpargv

    def test_generate_round_data(self):
        img, segm, seeds = io3d.datasets.generate_round_data(43)

        self.assertTrue(np.array_equal([43, 44, 45], img.shape))

    def test_generate_face_2d(self):
        img = io3d.datasets.generate_face()

        self.assertTrue(np.array_equal([32, 32], img.shape))
        self.assertEqual(img[0, 0], False, "In the corner should be False")
        self.assertEqual(img[15, 15], True, "In the middle of face should be True")

    def test_generate_face_3d(self):
        img = io3d.datasets.generate_face([5, 42, 34])

        self.assertTrue(np.array_equal([5, 42, 34], img.shape))
        self.assertEqual(img[2, 0, 0], False, "In the corner should be False")
        self.assertEqual(img[2, 21, 17], True, "In the middle of face should be True")

    def test_generate_donut(self):
        datap = io3d.datasets.generate_donut()
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]


        # self.assertTrue(np.array_equal([5, 42, 34], data3d.shape))
        self.assertEqual(segmentation[2, 0, 0], False, "In the corner should be False")
        self.assertTrue(np.array_equal(np.unique(segmentation), [0, 1, 2]), "Three labels in output segmentation")

    def test_generate_synghetic_liver(self):
        data3d, segm, voxelsize_mm, slab, seeds_liver, seeds_porta = io3d.datasets.generate_synthetic_liver()

        self.assertTrue(np.array_equal(data3d.shape, segm.shape))
        self.assertTrue(np.array_equal(data3d.shape, seeds_porta.shape))
        self.assertTrue(np.array_equal(data3d.shape, seeds_liver.shape))
        self.assertEqual(np.max(segm), 2, "Maximum label is 2")


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
""" Module for readin 3D dicom data
"""

# import funkcí z jiného adresáře
import sys
import os.path

# path_to_script = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src"))
# sys.path.append(os.path.join(path_to_script,
#                              "../extern/py3DSeedEditor/"))
# ys.path.append(os.path.join(path_to_script, "../extern/"))
# mport featurevector

import logging
logger = logging.getLogger(__name__)
import argparse

__author__ = 'mjirik'

import paramiko

host = "147.228.47.162"                    #hard-coded
port = 22
transport = paramiko.Transport((host, port))

username = "lisa_sftp"                #hard-coded
username = "lisa_normal"                #hard-coded
password = "L154.sftp"                #hard-coded
# username = "mjirik"                #hard-coded
# password = "titanic"                #hard-coded
transport.connect(username = username, password = password)

sftp = paramiko.SFTPClient.from_transport(transport)

import sys
path = './' + sys.argv[1]    #hard-coded
localpath = sys.argv[1]
sftp.put(localpath, path)

sftp.close()
transport.close()
print('Upload done.')






#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module is provides funcions for dict lists and functions processing
"""
import logging
logger = logging.getLogger(__name__)
import collections
import inspect
import copy
import numpy as np


def get_default_args(obj):
    if ("__init__" in dir(obj)):
        if inspect.isfunction(obj.__init__) or inspect.ismethod(obj.__init__):
            argspec = inspect.getargspec(obj.__init__)
        else:
            argspec = inspect.getargspec(obj)
    else:
        argspec = inspect.getargspec(obj)

    args = argspec.args[1:]
    defaults = argspec.defaults
    dc = collections.OrderedDict(zip(args, defaults))
    return dc

def subdict(dct, keys):
    if type(dct) == collections.OrderedDict:
        p = collections.OrderedDict()
    else:
        p = {}
    for key, value in dct.items():
        if key in keys:
            p[key] = value
    # p = {key: value for key, value in dct.items() if key in keys}
    return p


def list_filter(lst, startswith=None, notstartswith=None,
                contain=None, notcontain=None):
    """ Keep in list items according to filter parameters.

    :param lst: item list
    :param startswith: keep items starting with
    :param notstartswith: remove items starting with
    :return:
    """
    keeped = []
    for item in lst:
        keep = False
        if startswith is not None:
            if item.startswith(startswith):
                keep = True
        if notstartswith is not None:
            if not item.startswith(notstartswith):
                keep = True
        if contain is not None:
            if contain in item:
                keep = True
        if notcontain is not None:
            if not notcontain in item:
                keep = True

        if keep:
            keeped.append(item)
    return keeped

def kick_from_dict(dct, keys):
    if type(dct) == collections.OrderedDict:
        p = collections.OrderedDict()
    else:
        p = {}
    for key, value in dct.items():
        if key not in keys:
            p[key] = value

    # p = {key: value for key, value in dct.items() if key not in keys}
    return p

def split_dict(dct, keys):
    """
    Split dict into two subdicts based on keys.

    :param dct:
    :param keys:
    :return: dict_in, dict_out
    """
    if type(dct) == collections.OrderedDict:
        dict_in = collections.OrderedDict()
        dict_out = collections.OrderedDict()
    else:
        dict_in = {}
        dict_out = {}

    for key, value in dct.items:
        if key in keys:
            dict_in[key] = value
        else:
            dict_out[key] = value
    return dict_in, dict_out

def recursive_update(d, u):
    """
    Dict recursive update.

    Based on Alex Martelli code on stackoverflow
    http://stackoverflow.com/questions/3232943/update-value-of-a-nested-dictionary-of-varying-depth?answertab=votes#tab-top

    :param d: dict to update
    :param u: dict with new data
    :return:
    """
    for k, v in u.iteritems():
        if isinstance(v, collections.Mapping):
            r = recursive_update(d.get(k, {}), v)
            d[k] = r
        else:
            d[k] = u[k]
    return d

from collections import Mapping
from operator import add

_FLAG_FIRST = object()

def flatten_dict_join_keys(dct, join_symbol=" "):
    """ Flatten dict with defined key join symbol.

    :param dct: dict to flatten
    :param join_symbol: default value is " "
    :return:
    """
    return dict( flatten_dict(dct, join=lambda a,b:a+join_symbol+b) )


def flatten_dict(dct, separator=None, join=add, lift=lambda x:x):
    """

    Based on ninjagecko code on stackoveflow
    http://stackoverflow.com/questions/6027558/flatten-nested-python-dictionaries-compressing-keys

    :param dct: dict to flatten
    :param separator: use preset values for join and lift.
    Use empty list or tuple [], () for key hierarchy stored in list.
    If simple_mode is string it is used as a separator.
    :param join: join operation. To join keys with '_' use join=lambda a,b:a+'_'+b
    :param lift:  to have all hierarchy keys in lise use lift=lambda x:(x,))
    :return:

    For all keys from above hierarchy in list use:
    dict( flattenDict(testData, lift=lambda x:(x,)) )

    For all keys from abve hierarchy separated by '_' use:
    dict( flattenDict(testData, join=lambda a,b:a+'_'+b) )
    """

    if type(separator) is str:
        join = lambda a, b: a + separator + b
    elif type(separator) in (list, tuple):
        lift = lambda x:(x,)

    results = []
    def visit(subdict, results, partialKey):
        for k,v in subdict.items():
            newKey = lift(k) if partialKey==_FLAG_FIRST else join(partialKey,lift(k))
            if isinstance(v,Mapping):
                visit(v, results, newKey)
            else:
                results.append((newKey,v))
    visit(dct, results, _FLAG_FIRST)
    return results


def list_contains(list_of_strings, substring, return_true_false_array=False):
    """ Get strings in list which contains substring.

    """
    key_tf = [keyi.find(substring) != -1 for keyi in list_of_strings]
    if return_true_false_array:
        return key_tf
    keys_to_remove = list_of_strings[key_tf]
    return keys_to_remove


def df_drop_duplicates(df, ignore_key_pattern="time"):
    """
    Drop duplicates from dataframe ignore columns with keys containing defined pattern.

    :param df:
    :param noinfo_key_pattern:
    :return:
    """

    keys_to_remove = list_contains(df.keys(), ignore_key_pattern)
    #key_tf = [key.find(noinfo_key_pattern) != -1 for key in df.keys()]
    # keys_to_remove
    # remove duplicates
    ks = copy.copy(list(df.keys()))
    for key in keys_to_remove:
        ks.remove(key)

    df = df.drop_duplicates(ks)
    return df


def ndarray_to_list_in_structure(item, squeeze=True):
    """ Change ndarray in structure of lists and dicts into lists.
    """
    tp = type(item)

    if tp == np.ndarray:
        if squeeze:
            item = item.squeeze()
        item = item.tolist()
    elif tp == list:
        for i in range(len(item)):
            item[i] = ndarray_to_list_in_structure(item[i])
    elif tp == dict:
        for lab in item:
            item[lab] = ndarray_to_list_in_structure(item[lab])

    return item


def dict_find_key(dd, value):
    """ Find first suitable key in dict.

    :param dd:
    :param value:
    :return:
    """
    key = next(key for key, val in dd.items() if val == value)
    return key


def sort_list_of_dicts(lst_of_dct, keys, reverse=False, **sort_args):
    """
    Sort list of dicts by one or multiple keys.

    If the key is not available, sort these to the end.

    :param lst_of_dct: input structure. List of dicts.
    :param keys: one or more keys in list
    :param reverse:
    :param sort_args:
    :return:
    """

    if type(keys) != list:
        keys = [keys]
    # dcmdir = lst_of_dct[:]
    # lst_of_dct.sort(key=lambda x: [x[key] for key in keys], reverse=reverse, **sort_args)
    lst_of_dct.sort(key=lambda x: [((False, x[key]) if key in x else (True, 0)) for key in keys], reverse=reverse, **sort_args)
    return lst_of_dct

def ordered_dict_to_dict(config):
    """
    Use dict instead of ordered dict in structure.
    """

    if type(config) == collections.OrderedDict:
        config = dict(config)
    if type(config) == list:
        for i in range(0, len(config)):
            config[i] = ordered_dict_to_dict(config[i])
    elif type(config) == dict:
        for key in config:
            config[key] = ordered_dict_to_dict(config[key])

    return config


# def struct_to_yaml(cfg):
#     """
#     write complex struct with dicts and lists into yaml
#     :param cfg:
#     :return:
#     """
#     import yaml
#     # convert values to json
#     isconverted = {}
#     for key, value in cfg.iteritems():
#         if type(value) in (str, int, float, bool):
#
#             isconverted[key] = False
#             if type(value) is str:
#                 pass
#
#         else:
#             isconverted[key] = True
#             cfg[key] = yaml.dump(value, default_flow_style=True)
#     return cfg




#! /usr/bin/python
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)
DeprecationWarning("Image_manipulation would be removed from io3d in the future. Use imma.image_manipulation.")
logger.warning("Image_manipulation would be removed from io3d in the future. Use imma.image_manipulation.")
from imma.image_manipulation import *

# import os.path
# import sys
# import numpy as np
# import scipy
# import scipy.ndimage
#
# from io3d import dili
#
# path_to_script = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(path_to_script, "../extern/sed3"))
#
#
# def select_labels(segmentation, labels, slab=None):
#     """
#     return ndimage with zeros and ones based on input labels
#
#     :param segmentation: 3D ndimage
#     :param labels: labels to select
#     :param slab: dictionary{string_label: numeric_label}. Allow to use
#     string labels if it is defined
#     :return:
#     """
#
#     if slab is not None:
#         labels = get_nlabels(slab, labels)
#
#     if type(labels) not in (list, np.ndarray):
#         labels = [labels]
#
#     ds = np.zeros(segmentation.shape, np.bool)
#     for lab in labels:
#         dadd = (segmentation == lab)
#
#         ds = ds | dadd
#     if len(labels) == 0:
#         logger.warning("Labels not found in slab.")
#
#     return ds
#
#
# def get_nlabels(slab, labels, labels_meta=None, return_mode="num", return_first=False):
#     """
#     Get one or more labels, create a new one if necessary and return its numeric value.
#
#     Look at the get_nlabel function for more details.
#
#     :param slab:
#     :param labels:
#     :param labels_meta:
#     :param return_mode: "num" or "str" or "both". Both means (numlabel, strlabel).
#     :param return_first: Return just first found label
#     :return:
#     """
#
#     if type(labels) not in (list, np.ndarray):
#         labels = [labels]
#         labels_meta = [labels_meta]
#         return_first = True
#
#     if labels_meta is None:
#         labels_meta = [None] * len(labels)
#
#     nlabels = []
#     for label, label_meta in zip(labels, labels_meta):
#         nlab = get_nlabel(slab, label, label_meta, return_mode=return_mode)
#         nlabels.append(nlab)
#
#     if return_first:
#         nlabels = nlabels[0]
#     return nlabels
#
#
# def get_nlabel(slab, label, label_meta=None, return_mode="num"):
#     """
#     Add label if it is necessery and return its numeric value.
#
#     If "new" keyword is used and no other information is provided, the max + 1 label is created.
#     If "new" keyword is used and additional numeric info is provided, the number is used also as a key.
#     :param return_mode: Set requested label return type. "int", "num", "numeric" or "str" or "both".
#     "both" means (numlabel, strlabel).
#     :param label: string, number or "new"
#     :param label_meta: string, number or "new
#     :return:
#     """
#     numlabel = None
#     strlabel = None
#     if type(label) == str:
#         if label_meta is None:
#             if label not in slab.keys():
#                 free_numeric_label = np.max(list(slab.values())) + 1
#                 if label == "new":
#                     label = str(free_numeric_label)
#                 slab[label] = free_numeric_label
#                 strlabel = label
#                 numlabel = slab[label]
#             else:
#                 strlabel = label
#                 numlabel = slab[label]
#         else:
#             if label == "new":
#                 label = str(label_meta)
#             update_slab(slab, label_meta, label)
#             strlabel = label
#             numlabel = label_meta
#     else:
#         # it is numeric
#         if label_meta is None:
#             if label not in list(slab.values()):
#                 update_slab(slab, label, str(label))
#                 strlabel = str(label)
#             else:
#                 strlabel = dili.dict_find_key(slab, label)
#
#             numlabel = label
#
#         else:
#             if label_meta == "new":
#                 label_meta = str(label)
#             update_slab(slab, label, label_meta)
#             strlabel = label_meta
#             numlabel = label
#             # return label
#
#     if return_mode in ("num", "int", "numeric"):
#         return numlabel
#     elif return_mode == "str":
#         return strlabel
#     elif return_mode == "both":
#         return numlabel, strlabel
#     else:
#         logger.error("Unknown return_mode: " + str(return_mode))
#
#
# def update_slab(slab, numeric_label, string_label):
#     """ Add label to segmentation label dictionary if it is not there yet.
#
#     :param numeric_label:
#     :param string_label:
#     :return:
#     """
#
#     slab_tmp = {string_label: numeric_label}
#     slab.update(slab_tmp)
#     # slab = slab_tmp
#     logger.debug('self.slab')
#     logger.debug(str(slab))
#
#
# def add_slab_label_carefully2(slab, numeric_label, string_label):
#     """ Add label to slab if it is not there yet.
#
#     :param numeric_label:
#     :param string_label:
#     :return:
#     """
#     # todo implement
#     # if numeric_label in
#     pass
#
#
# def add_missing_labels(segmentation, slab):
#     labels = np.unique(segmentation)
#     get_nlabels(slab, labels)
#
#
# def add_slab_label_carefully(slab, numeric_label, string_label):
#     """ Add label to slab if it is not there yet.
#
#     :param numeric_label:
#     :param string_label:
#     :return:
#     """
#     slab_tmp = {string_label: numeric_label}
#     slab_tmp.update(slab)
#     slab = slab_tmp
#     logger.debug('self.slab')
#     logger.debug(str(slab))
#
#
# def add_missing_labels(segmentation, slab):
#     labels = np.unique(segmentation)
#     get_nlabels(slab, labels)
#
#
# class SparseMatrix():
#     def __init__(self, ndarray):
#         self.coordinates = ndarray.nonzero()
#         self.shape = ndarray.shape
#         self.values = ndarray[self.coordinates]
#         self.dtype = ndarray.dtype
#         self.sparse = True
#
#     def todense(self):
#         dense = np.zeros(self.shape, dtype=self.dtype)
#         dense[self.coordinates[:]] = self.values
#         return dense
#
#
# def isSparseMatrix(obj):
#     if obj.__class__.__name__ == 'SparseMatrix':
#         return True
#     else:
#         return False
#
#
# # import sed3
#
# def manualcrop(data):  # pragma: no cover
#
#     try:
#         from pysegbase import seed_editor_qt
#     except:
#         logger.warning("Deprecated of pyseg_base as submodule")
#         import seed_editor_qt
#
#     pyed = seed_editor_qt.QTSeedEditor(data, mode='crop')
#     pyed.exec_()
#     # pyed = sed3.sed3(data)
#     # pyed.show()
#     nzs = pyed.seeds.nonzero()
#     crinfo = [
#         [np.min(nzs[0]), np.max(nzs[0])],
#         [np.min(nzs[1]), np.max(nzs[1])],
#         [np.min(nzs[2]), np.max(nzs[2])],
#     ]
#     data = crop(data, crinfo)
#     return data, crinfo
#
#
# def crop(data, crinfo):
#     """
#     Crop the data.
#
#     crop(data, crinfo)
#
#     :param crinfo: min and max for each axis - [[minX, maxX], [minY, maxY], [minZ, maxZ]]
#
#     """
#     crinfo = fix_crinfo(crinfo)
#     return data[
#            __int_or_none(crinfo[0][0]):__int_or_none(crinfo[0][1]),
#            __int_or_none(crinfo[1][0]):__int_or_none(crinfo[1][1]),
#            __int_or_none(crinfo[2][0]):__int_or_none(crinfo[2][1])
#            ]
#
#
# def __int_or_none(number):
#     if number is not None:
#         number = int(number)
#     return number
#
#
# def combinecrinfo(crinfo1, crinfo2):
#     """
#     Combine two crinfos. First used is crinfo1, second used is crinfo2.
#     """
#     crinfo1 = fix_crinfo(crinfo1)
#     crinfo2 = fix_crinfo(crinfo2)
#
#     crinfo = [
#         [crinfo1[0][0] + crinfo2[0][0], crinfo1[0][0] + crinfo2[0][1]],
#         [crinfo1[1][0] + crinfo2[1][0], crinfo1[1][0] + crinfo2[1][1]],
#         [crinfo1[2][0] + crinfo2[2][0], crinfo1[2][0] + crinfo2[2][1]]
#     ]
#
#     return crinfo
#
#
# def crinfo_from_specific_data(data, margin=0):
#     """
#     Create crinfo of minimum orthogonal nonzero block in input data.
#
#     :param data: input data
#     :param margin: add margin to minimum block
#     :return:
#     """
#     # hledáme automatický ořez, nonzero dá indexy
#     logger.debug('crinfo')
#     logger.debug(str(margin))
#     nzi = np.nonzero(data)
#     logger.debug(str(nzi))
#
#     if np.isscalar(margin):
#         margin = [margin] * 3
#
#     x1 = np.min(nzi[0]) - margin[0]
#     x2 = np.max(nzi[0]) + margin[0] + 1
#     y1 = np.min(nzi[1]) - margin[0]
#     y2 = np.max(nzi[1]) + margin[0] + 1
#     z1 = np.min(nzi[2]) - margin[0]
#     z2 = np.max(nzi[2]) + margin[0] + 1
#
#     # ošetření mezí polí
#     if x1 < 0:
#         x1 = 0
#     if y1 < 0:
#         y1 = 0
#     if z1 < 0:
#         z1 = 0
#
#     if x2 > data.shape[0]:
#         x2 = data.shape[0] - 1
#     if y2 > data.shape[1]:
#         y2 = data.shape[1] - 1
#     if z2 > data.shape[2]:
#         z2 = data.shape[2] - 1
#
#     # ořez
#     crinfo = [[x1, x2], [y1, y2], [z1, z2]]
#     return crinfo
#
#
# def uncrop(data, crinfo, orig_shape, resize=False, outside_mode="constant", cval=0):
#     """
#     Put some boundary to input image.
#
#
#     :param data: input data
#     :param crinfo: array with minimum and maximum index along each axis
#         [[minX, maxX],[minY, maxY],[minZ, maxZ]]. If crinfo is None, the whole input image is placed into [0, 0, 0].
#         If crinfo is just series of three numbers, it is used as an initial point for input image placement.
#     :param orig_shape: shape of uncropped image
#     :param resize: True or False (default). Usefull if the data.shape does not fit to crinfo shape.
#     :param outside_mode: 'constant', 'nearest'
#     :return:
#     """
#
#     if crinfo is None:
#         crinfo = list(zip([0] * data.ndim, orig_shape))
#     elif np.asarray(crinfo).size == data.ndim:
#         crinfo = list(zip(crinfo, np.asarray(crinfo) + data.shape))
#
#     crinfo = fix_crinfo(crinfo)
#     data_out = np.ones(orig_shape, dtype=data.dtype) * cval
#
#     # print 'uncrop ', crinfo
#     # print orig_shape
#     # print data.shape
#     if resize:
#         data = resize_to_shape(data, crinfo[:, 1] - crinfo[:, 0])
#
#     startx = np.round(crinfo[0][0]).astype(int)
#     starty = np.round(crinfo[1][0]).astype(int)
#     startz = np.round(crinfo[2][0]).astype(int)
#
#     data_out[
#     # np.round(crinfo[0][0]).astype(int):np.round(crinfo[0][1]).astype(int)+1,
#     # np.round(crinfo[1][0]).astype(int):np.round(crinfo[1][1]).astype(int)+1,
#     # np.round(crinfo[2][0]).astype(int):np.round(crinfo[2][1]).astype(int)+1
#     startx:startx + data.shape[0],
#     starty:starty + data.shape[1],
#     startz:startz + data.shape[2]
#     ] = data
#
#     if outside_mode == "nearest":
#         # for ax in range(data.ndims):
#         # ax = 0
#
#         # copy border slice to pixels out of boundary - the higher part
#         for ax in range(data.ndim):
#             # the part under the crop
#             start = np.round(crinfo[ax][0]).astype(int)
#             slices = [slice(None), slice(None), slice(None)]
#             slices[ax] = start
#             repeated_slice = np.expand_dims(data_out[slices], ax)
#             append_sz = start
#             if append_sz > 0:
#                 tile0 = np.repeat(repeated_slice, append_sz, axis=ax)
#                 slices = [slice(None), slice(None), slice(None)]
#                 slices[ax] = slice(None, start)
#                 # data_out[start + data.shape[ax] : , :, :] = tile0
#                 data_out[slices] = tile0
#                 # plt.imshow(np.squeeze(repeated_slice))
#                 # plt.show()
#
#             # the part over the crop
#             start = np.round(crinfo[ax][0]).astype(int)
#             slices = [slice(None), slice(None), slice(None)]
#             slices[ax] = start + data.shape[ax] - 1
#             repeated_slice = np.expand_dims(data_out[slices], ax)
#             append_sz = data_out.shape[ax] - (start + data.shape[ax])
#             if append_sz > 0:
#                 tile0 = np.repeat(repeated_slice, append_sz, axis=ax)
#                 slices = [slice(None), slice(None), slice(None)]
#                 slices[ax] = slice(start + data.shape[ax], None)
#                 # data_out[start + data.shape[ax] : , :, :] = tile0
#                 data_out[slices] = tile0
#                 # plt.imshow(np.squeeze(repeated_slice))
#                 # plt.show()
#
#     return data_out
#
#
# def fix_crinfo(crinfo, to='axis'):
#     """
#     Function recognize order of crinfo and convert it to proper format.
#     """
#
#     crinfo = np.asarray(crinfo)
#     if crinfo.shape[0] == 2:
#         crinfo = crinfo.T
#
#     return crinfo
#
#
# def get_one_biggest_object(data):
#     """ Return biggest object """
#     lab, num = scipy.ndimage.label(data)
#     # print ("bum = "+str(num))
#
#     maxlab = max_area_index(lab, num)
#
#     data = (lab == maxlab)
#     return data
#
#
# def max_area_index(labels, num):
#     """
#     Return index of maxmum labeled area
#     """
#     mx = 0
#     mxi = -1
#     for l in range(1, num + 1):
#         mxtmp = np.sum(labels == l)
#         if mxtmp > mx:
#             mx = mxtmp
#             mxi = l
#
#     return mxi
#
#
# def resize_to_shape(data, shape, zoom=None, mode='nearest', order=0):
#     """Resize input data to specific shape.
#
#     :param data: input 3d array-like data
#     :param shape: shape of output data
#     :param zoom: zoom is used for back compatibility
#     :mode: default is 'nearest'
#     """
#     # @TODO remove old code in except part
#
#     if np.array_equal(data.shape, shape):
#         return data
#
#     try:
#         # rint 'pred vyjimkou'
#         # aise Exception ('test without skimage')
#         # rint 'za vyjimkou'
#         import skimage
#         import skimage.transform
#         # Now we need reshape  seeds and segmentation to original size
#
#         segm_orig_scale = skimage.transform.resize(
#             data, shape, order=order,
#             preserve_range=True
#         )
#
#         segmentation = segm_orig_scale
#         logger.debug('resize to orig with skimage')
#     except Exception:
#         logger.warning("Resize by scipy will be removed in the future")
#         import scipy
#         import scipy.ndimage
#         dtype = data.dtype
#         if zoom is None:
#             zoom = np.asarray(data.shape).astype(np.double) / shape
#
#         segm_orig_scale = scipy.ndimage.zoom(
#             data,
#             1.0 / zoom,
#             mode=mode,
#             order=order
#         ).astype(dtype)
#         logger.debug('resize to orig with scipy.ndimage')
#
#         # @TODO odstranit hack pro oříznutí na stejnou velikost
#         # v podstatě je to vyřešeno, ale nechalo by se to dělat elegantněji v zoom
#         # tam je bohužel patrně bug
#         # rint 'd3d ', self.data3d.shape
#         # rint 's orig scale shape ', segm_orig_scale.shape
#         shp = [
#             np.min([segm_orig_scale.shape[0], shape[0]]),
#             np.min([segm_orig_scale.shape[1], shape[1]]),
#             np.min([segm_orig_scale.shape[2], shape[2]]),
#         ]
#         # elf.data3d = self.data3d[0:shp[0], 0:shp[1], 0:shp[2]]
#         # mport ipdb; ipdb.set_trace() # BREAKPOINT
#
#         segmentation = np.zeros(shape, dtype=dtype)
#         segmentation[
#         0:shp[0],
#         0:shp[1],
#         0:shp[2]] = segm_orig_scale[0:shp[0], 0:shp[1], 0:shp[2]]
#
#         del segm_orig_scale
#     return segmentation
#
#
# def resize_to_mm(data3d, voxelsize_mm, new_voxelsize_mm, mode='edge', order=1):
#     """
#     Function can resize data3d or segmentation to specifed voxelsize_mm
#     :new_voxelsize_mm: requested voxelsize. List of 3 numbers, also
#         can be a string 'orig', 'orig*2' and 'orig*4'.
#
#     :voxelsize_mm: size of voxel
#     :mode: default is 'edge'. Modes match the behaviour of numpy.pad
#     """
#
#     if new_voxelsize_mm is 'orig':
#         new_voxelsize_mm = np.array(voxelsize_mm)
#
#     elif new_voxelsize_mm is 'orig*2':
#         new_voxelsize_mm = np.array(voxelsize_mm) * 2
#     elif new_voxelsize_mm is 'orig*4':
#         new_voxelsize_mm = np.array(voxelsize_mm) * 4
#         # vx_size = np.array(metadata['voxelsize_mm']) * 4
#
#     zoom = voxelsize_mm / (1.0 * np.array(new_voxelsize_mm))
#     # data3d_res = scipy.ndimage.zoom(
#     #     data3d,
#     #     zoom,
#     #     mode=mode,
#     #     order=order
#     # ).astype(data3d.dtype)
#
#     # probably better implementation
#     new_shape = data3d.shape * zoom
#     import skimage.transform
#     # Now we need reshape  seeds and segmentation to original size
#
#     data3d_res2 = skimage.transform.resize(
#         data3d, new_shape, order=order,
#         mode=mode,
#         preserve_range=True
#     ).astype(data3d.dtype)
#
#     return data3d_res2
#
#
# def select_objects_by_seeds(binar_data, seeds, ignore_background_seeds=True, background_label=0):
#     """
#     Get N biggest objects from the selection or the object with seed.
#
#     :param binar_data:  binar ndarray
#     :param seeds: ndarray. Objects on non zero positions are returned
#     :return:
#     """
#
#     labeled_data, length = scipy.ndimage.label(binar_data)
#     selected_labels = list(np.unique(labeled_data[seeds > 0]))
#     # selected_labels.pop(0)
#     # pop the background label
#     output = np.zeros_like(binar_data)
#     for label in selected_labels:
#         selection = labeled_data == label
#         # copy from input image to output. If there will be seeds in background, the 0 is copied
#         if ignore_background_seeds and (binar_data[selection][0] == background_label):
#             pass
#         else:
#             # output[selection] = binar_data[selection]
#             output[selection] = 1
#     # import sed3
#     # ed =sed3.sed3(labeled_data, contour=output, seeds=seeds)
#     # ed.show()
#     return output
#
#
# def rotate(data3d, phi_deg, theta_deg=None, phi_axes=(1, 2), theta_axes=(0, 1), order=1, **kwargs):
#     """
#     Rotate 3D data by use angle and its axes or two angles.
#
#     :param data3d: ndimage 3D
#     :param phi_deg: deg
#     :param phi_axes: deg
#     :param theta_deg: deg
#     :param theta_axes: deg
#     :param order: optional, int. Default is 1. The order of the spline interpolation. See scipy for more details.
#     :param kwargs: See scipy.ndimage.interpolation.rotate for more options
#     :return:
#     """
#
#     data3d = scipy.ndimage.interpolation.rotate(data3d, phi_deg, phi_axes, order=order, **kwargs)
#     if theta_deg is not None:
#         data3d = scipy.ndimage.interpolation.rotate(data3d, theta_deg, theta_axes, order=order, **kwargs)
#     return data3d
#     # segmentation = scipy.ndimage.interpolation.rotate(segmentation, angle, axes)
#     # seeds = scipy.ndimage.interpolation.rotate(seeds, angle, axes)
#
#     return data3d
#
#
# def random_rotate_paramteres():
#     """
#     Rotate data3d, segmentation and seeds with random rotation
#     :return:
#     """
#     xi1 = np.random.rand()
#     xi2 = np.random.rand()
#
#     # theta = np.arccos(np.sqrt(1.0-xi1))
#     theta = np.arccos(1.0 - (xi1 * 1))
#     phi = xi2 * 2 * np.pi
#
#     # xs = np.sin(theta) * np.cos(phi)
#     # ys = np.sin(theta) * np.sin(phi)
#     # zs = np.cos(theta)
#
#     phi_deg = np.degrees(phi)
#     theta_deg = np.degrees(theta)
#
#     return phi_deg, theta_deg
#     # TODO independent on voxlelsize (2016-techtest-rotate3d.ipynb)
#
#
# def as_seeds_inds(seeds, datashape):
#     sh1 = datashape
#     sh2 = np.asarray(seeds).shape
#     if np.array_equal(sh1, sh2):
#         seeds_inds = np.nonzero(seeds)
#     else:
#         seeds_inds = seeds
#     return seeds_inds
#
#
# def squeeze_labels(segmentation, try_inplace=True):
#     """
#     Squeeze all labels to int numbers starting from 1
#
#     :param segmentation: labeled image
#     :param try_inplace: try to compute inplace
#     :return:
#     """
#     un = np.unique(segmentation)
#
#     if try_inplace:
#         inplace_possible = True
#         for new_level, level in enumerate(un):
#             # if i == level:
#             #     continue
#             if (new_level > level) and (new_level in un):
#                 # we will rewrite old
#                 inplace_possible = False
#                 break
#     else:
#         inplace_possible = False
#
#     if inplace_possible:
#         output = segmentation
#     else:
#         import copy
#         output = copy.copy(segmentation)
#
#     for new_level, level in enumerate(un):
#         output[segmentation == level] = new_level
#
#     return output
#
#
# def distance_segmentation(seeds, method="edt"):
#     """
#     Separates space based on distance to seeds.
#     :param seeds: ndarray with zeros for background and labeled seeds.
#     :param method: scipy.ndimage.distance_transform function.
#     The `distance_transform_edt` is used if is set to "edt"
#     :return:
#     """
#     if method is "edt":
#         dst_transform = scipy.ndimage.distance_transform_edt
#     else:
#         dst_transform = method
#
#     inds = dst_transform(seeds==0, return_indices=True, return_distances=False)
#
#     lin_inds = []
#     for one_ax in inds:
#         lin_inds.append(one_ax.ravel())
#
#     segm = seeds[lin_inds].reshape(seeds.shape)
#     return segm
#
#
#
#     pass
#




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Deprecation decorator, taken from https://gist.github.com/kgriffs/8202106"""

import functools
import inspect
import warnings


# NOTE: We don't want our deprecations to be ignored by default, so create our own type.
class DeprecatedWarning(UserWarning):
    pass


def deprecated(instructions):
    """
    Flags a method as deprecated.

    :param instructions: A human-friendly string of instructions, such as: 'Please migrate to add_proxy() ASAP.'
    :return: DeprecatedWarning
    """
    def decorator(func):
        """This is a decorator which can be used to mark functions as deprecated.

        It will result in a warning being emitted when the function is used.
        """
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            message = 'Call to deprecated function {}. {}'.format(func.__name__,
                                                                  instructions)
            frame = inspect.currentframe().f_back
            warnings.warn_explicit(message,
                                   category=DeprecatedWarning,
                                   filename=inspect.getfile(frame.f_code),
                                   lineno=frame.f_lineno)
            return func(*args, **kwargs)
        return wrapper
    return decorator


def main():
    pass


if __name__ == '__main__':
    main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2015 mjirik <mjirik@mjirik-Latitude-E6520>
#
# Distributed under terms of the MIT license.

"""

"""

import logging
logger = logging.getLogger(__name__)
import argparse
import struct
import numpy as np
import os.path as op


def read(filename):
    return read_iv(filename)


def __iv_read_header(f):
    minX = __read_float(f)
    minY = __read_float(f)
    minZ = __read_float(f)
    maxX = __read_float(f)
    maxY = __read_float(f)
    maxZ = __read_float(f)
    numVerts = __read_uint(f)
    numCells = __read_uint(f)
    dimX = __read_uint(f)
    dimY = __read_uint(f)
    dimZ = __read_uint(f)
    originX = __read_float(f)
    originY = __read_float(f)
    originZ = __read_float(f)
    spanXorig = __read_float(f)
    spanYorig = __read_float(f)
    spanZorig = __read_float(f)

    # print(dimX * dimY * dimZ, '   ', numVerts)
    if dimX * dimY * dimZ != numVerts:
        logger.error('numVerts is not consistent with dimX, dimY, dimZ')
        print("error")

    spanX  = (maxX - minX)/(dimX -1)
    spanY  = (maxY - minY)/(dimY -1)
    spanZ  = (maxZ - minZ)/(dimZ -1)

    if spanX != spanXorig or \
            spanY != spanYorig or\
            spanZ != spanZorig:
        logger.warning('span in rawiv header is not consistent')
        print('error span')

    return minX , minY , minZ , maxX , maxY , maxZ , numVerts , numCells , dimX\
     , dimY , dimZ , originX , originY , originZ , spanX, spanY, \
     spanZ


def read_iv(filename):
    filename = op.expanduser(filename)
    with open(filename, "rb") as f:
        head = __iv_read_header(f)

        minX, minY, minZ, maxX, maxY, maxZ, numVerts, numCells, dimX,\
            dimY, dimZ, originX, originY, originZ, spanX,\
            spanY, spanZ = head
        # check



        data = f.read()
        i = 0

        print(len(data))
        print(numVerts)
        bytes_per_vertex = len(data)//numVerts

        if bytes_per_vertex == 1:
            nptype = 'uint8'
            pctype = '>B'
        elif bytes_per_vertex == 2:
            nptype = 'uint16'
            pctype = '>H'
        elif bytes_per_vertex == 4:
            nptype = 'float'
            pctype = '>f'

        data3d = np.zeros([dimX, dimY, dimZ], dtype=nptype)


        for z in range(0, dimZ):
            for y in range(0, dimY):
                for x in range(0, dimX):
                    dataraw = data[i:(i+bytes_per_vertex)]
                    data3d[x,y,z] = struct.unpack(pctype, dataraw)[0]
                    i = i + 1

        metadata = {
            'minX':      minX,
            'minY':      minY,
            'minZ':      minZ,
            'maxX':      maxX,
            'maxY':      maxY,
            'maxZ':      maxZ,
            'numVerts':  numVerts,
            'numCells':  numCells,
            'dimX':      dimX,
            'dimY':      dimY,
            'dimZ':      dimZ,
            'originX':   originX,
            'originY':   originY,
            'originZ':   originZ,
            'spanX':     spanX,
            'spanY':     spanY,
            'spanZ':     spanZ,
            'voxelsize_mm': [spanX, spanY, spanZ]
        }


    return data3d, metadata

def __read_float(f):
        data = f.read(4)
        number = struct.unpack('>f', data)
        number = number[0]
        return number


def __read_uint(f):
        data = f.read(4)
        number = struct.unpack('>I', data)
        number = number[0]
        return number


def __write_float(f, number):
        data = struct.pack('>f', number)
        f.write(data)


def __write_uint(f, number):
        data = struct.pack('>I', number)
        f.write(data)


def write(filename, data3d, metadata):
    data3d.shape[0]

    minX = 0
    minY = 0
    minZ = 0
    spanX = metadata['voxelsize_mm'][0]
    spanY = metadata['voxelsize_mm'][1]
    spanZ = metadata['voxelsize_mm'][2]
    dimX = data3d.shape[0]
    dimY = data3d.shape[1]
    dimZ = data3d.shape[2]
    numVerts = dimX * dimY * dimZ
    maxX = spanX * (dimX - 1) + minX
    maxY = spanY * (dimY - 1) + minY
    maxZ = spanZ * (dimZ - 1) + minZ
    numCells = (dimX - 1) * (dimY - 1) * (dimZ - 1)
    originX = 0
    originY = 0
    originZ = 0

    with open(filename, "wb") as f:
        __write_float(f, minX)
        __write_float(f, minY)
        __write_float(f, minZ)
        __write_float(f, maxX)
        __write_float(f, maxY)
        __write_float(f, maxZ)
        __write_uint(f, numVerts)
        __write_uint(f, numCells)
        __write_uint(f, dimX)
        __write_uint(f, dimY)
        __write_uint(f, dimZ)
        __write_float(f, originX)
        __write_float(f, originY)
        __write_float(f, originZ)
        __write_float(f, spanX)
        __write_float(f, spanY)
        __write_float(f, spanZ)

        bytes_per_vertex = 1
        pctype = '>B'

        for z in range(0, dimZ):
            for y in range(0, dimY):
                for x in range(0, dimX):
                    dataraw = struct.pack(pctype, data3d[x,y,z])
                    f.write(dataraw)

def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    parser.add_argument(
        '-i', '--inputfile',
        default=None,
        required=True,
        help='input file'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)



if __name__ == "__main__":
    main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Module for reading 3D dicom data"""

import logging
logger = logging.getLogger(__name__)

import argparse
import numpy as np
import h5py

import os.path
import sys

try:
    import dicom
    dicom.debug(False)
except ImportError:
    import pydicom as dicom
    dicom.config.debug(False)

# NOTE(mareklovci - 2018_05_13): Absolute imports are prefered in Python, so eg. "from io3d import tgz" should be used.
# https://www.python.org/dev/peps/pep-0008/#imports
from . import dcmreaddata as dcmr
from . import tgz
from . import misc
from . import dcmtools

# Decorator used for labeling old or unsuitable functions as 'deprecated'
from io3d.deprecation import deprecated



def read(datapath, qt_app=None, dataplus_format=True, gui=False, start=0, stop=None, step=1, convert_to_gray=True,
         series_number=None, dicom_expected=None, **kwargs):
    """Simple read function. Internally calls DataReader.Get3DData()"""
    dr = DataReader()
    return dr.Get3DData(datapath=datapath, qt_app=qt_app, dataplus_format=dataplus_format, gui=gui, start=start,
                        stop=stop, step=step, convert_to_gray=convert_to_gray, series_number=series_number,
                        use_economic_dtype=True, dicom_expected=dicom_expected  , **kwargs)


# NOTE(mareklovci): The same code was used in two functions, so according to DRY principle I cleaned it up.
def _metadata(image, datapath):
    """Function which returns metadata dict.

    :param image: image to get spacing from
    :param datapath: path to data
    :return: {'series_number': '', 'datadir': '', 'voxelsize_mm': ''}
    """
    metadata = {'series_number': 0, 'datadir': datapath}
    spacing = image.GetSpacing()
    metadata['voxelsize_mm'] = [
        spacing[2],
        spacing[0],
        spacing[1],
    ]
    return metadata


class DataReader:

    def __init__(self):
        self.overlay_fcn = None

    # noinspection PyAttributeOutsideInit,PyUnboundLocalVariable,PyPep8Naming
    def Get3DData(self, datapath, qt_app=None, dataplus_format=True, gui=False, start=0, stop=None, step=1,
                  convert_to_gray=True, series_number=None, use_economic_dtype=True, dicom_expected=None, **kwargs):
        """Returns 3D data and its metadata.

        # NOTE(:param qt_app:) If it is set to None (as default) all dialogs for series selection are performed in
        terminal. If qt_app is set to QtGui.QApplication() dialogs are in Qt.

        :param datapath: directory with input data
        :param qt_app: Dialog destination. If None (default) -> terminal, if 'QtGui.QApplication()' -> Qt
        :param dataplus_format: New data format. Metadata and data are returned in one structure.
        :param gui: True if 'QtGui.QApplication()' instead of terminal should be used
        :param int start: used for DicomReader, defines where 3D data reading should start
        :param int stop: used for DicomReader, defines where 3D data reading should stop
        :param int step: used for DicomReader, defines step for 3D data reading
        :param bool convert_to_gray: if True -> RGB is converted to gray
        :param int series_number: used in DicomReader, essential in metadata
        :param use_economic_dtype: if True, casts 3D data array to less space consuming dtype
        :param dicom_expected: set true if it is known that data is in dicom format. Set False to suppress
        dicom warnings.
        :return: tuple (data3d, metadata)
        """
        self.orig_datapath = datapath
        datapath = os.path.expanduser(datapath)

        if series_number is not None and type(series_number) != int:
            series_number = int(series_number)

        if not os.path.exists(datapath):
            logger.error("Path '" + datapath + "' does not exist")
            return
        if qt_app is None and gui is True:
            from PyQt4.QtGui import QApplication
            qt_app = QApplication(sys.argv)

        if type(datapath) is not str:
            datapath = str(datapath)
        datapath = os.path.normpath(datapath)

        self.start = start
        self.stop = stop
        self.step = step
        self.convert_to_gray = convert_to_gray
        self.series_number = series_number
        self.kwargs = kwargs
        self.qt_app = qt_app
        self.gui = gui

        if os.path.isfile(datapath):
            logger.debug('file read recognized')
            data3d, metadata = self.__ReadFromFile(datapath)

        elif os.path.exists(datapath):
            logger.debug('directory read recognized')
            data3d, metadata = self.__ReadFromDirectory(datapath=datapath, dicom_expected=dicom_expected)
            # datapath, start, stop, step, gui=gui, **kwargs)
        else:
            logger.error('Data path {} not found'.format(datapath))

        if convert_to_gray:
            if len(data3d.shape) > 3:
                # TODO: implement better rgb2gray
                data3d = data3d[:, :, :, 0]
        if use_economic_dtype:
            data3d = self.__use_economic_dtype(data3d)

        if dataplus_format:
            logger.debug('dataplus format')
            # metadata = {'voxelsize_mm': [1, 1, 1]}
            datap = metadata
            datap['data3d'] = data3d
            logger.debug('datap keys () : ' + str(datap.keys()))
            return datap
        else:
            return data3d, metadata

    # noinspection PyPep8Naming
    def __ReadFromDirectory(self, datapath, dicom_expected=None):
        """This function is actually the ONE, which reads 3D data from file

        :param datapath: path to file
        :return: tuple (data3d, metadata)
        """
        start = self.start
        stop = self.stop
        step = self.step
        kwargs = self.kwargs
        gui = self.gui

        if (dicom_expected is not False) and (dcmr.is_dicom_dir(datapath)):  # reading dicom
            logger.debug('Dir - DICOM')
            logger.debug("dicom_expected " + str(dicom_expected))
            reader = dcmr.DicomReader(datapath,
                                      series_number=self.series_number,
                                      gui=gui,
                                      **kwargs)  # qt_app=None, gui=True)
            data3d = reader.get_3Ddata(start, stop, step)
            metadata = reader.get_metaData()
            metadata['series_number'] = reader.series_number
            metadata['datadir'] = datapath
            self.overlay_fcn = reader.get_overlay
        else:  # reading image sequence
            logger.debug('Dir - Image sequence')

            logger.debug('Getting list of readable files...')
            flist = []
            try:
                import SimpleITK as Sitk
            except ImportError as e:
                logger.error("Unable to import SimpleITK. On Windows try version 1.0.1")
            for f in os.listdir(datapath):
                try:
                    Sitk.ReadImage(os.path.join(datapath, f))
                except Exception as e:
                    logger.warning("Cant load file: " + str(f))
                    logger.warning(e)
                    continue
                flist.append(os.path.join(datapath, f))
            flist.sort()

            logger.debug('Reading image data...')
            image = Sitk.ReadImage(flist)
            logger.debug('Getting numpy array from image data...')
            data3d = Sitk.GetArrayFromImage(image)
            metadata = _metadata(image, datapath)
        return data3d, metadata

    @staticmethod
    def __use_economic_dtype(data3d):
        """Casts 3D data array to less space consuming dtype.

        :param data3d: array of 3D data to reformat
        :return: reformated array
        """
        return misc.use_economic_dtype(data3d)

    # noinspection PyPep8Naming
    def __ReadFromFile(self, datapath):
        """Reads file and returns containing 3D data and its metadata.
        Supported formats: pklz, pkl, hdf5, idx, dcm, Dcm, dicom, bz2 and "raw files"

        :param datapath: path to file to read
        :return: tuple (data3d, metadata)
        """
        def _create_meta(_datapath):
            """Just simply returns some dict. This functions exists in order to keep DRY"""
            meta = {
                'series_number': 0,
                'datadir': _datapath
            }
            return meta

        path, ext = os.path.splitext(datapath)
        ext = ext[1:]
        if ext in ('pklz', 'pkl'):
            logger.debug('pklz format detected')
            from . import misc
            data = misc.obj_from_file(datapath, filetype='pkl')
            data3d = data.pop('data3d')
            # metadata must have series_number
            metadata = _create_meta(datapath)
            metadata.update(data)

        elif ext in ['hdf5']:
            from . import hdf5_io
            datap = hdf5_io.load_dict_from_hdf5(datapath)
            # datap = self.read_hdf5(datapath)
            data3d = datap.pop('data3d')

            # back compatibility
            if 'metadata' in datap.keys():
                datap = datap['metadata']
            # metadata must have series_number
            metadata = _create_meta(datapath)
            metadata.update(datap)

        elif ext in ['idx']:
            from . import idxformat
            idxreader = idxformat.IDXReader()
            data3d, metadata = idxreader.read(datapath)
        elif ext in ['dcm', 'DCM', 'dicom']:
            data3d, metadata = self._read_with_sitk(datapath)
            metadata = self._fix_sitk_bug(datapath, metadata)
        elif ext in ["bz2"]:
            new_datapath = tgz.untar(datapath)
            data3d, metadata = self.__ReadFromDirectory(new_datapath)
        else:
            logger.debug('file format "' + str(ext) + '"')
            # reading raw file
            data3d, metadata = self._read_with_sitk(datapath)
        return data3d, metadata

    @staticmethod
    def _read_with_sitk(datapath):
        """Reads file using SimpleITK. Returns array of pixels (image located in datapath) and its metadata.

        :param datapath: path to file (img or dicom)
        :return: tuple (data3d, metadata), where data3d is array of pixels
        """
        try:
            import SimpleITK as Sitk
        except ImportError as e:
            logger.error("Unable to import SimpleITK. On Windows try version 1.0.1")
        image = Sitk.ReadImage(datapath)
        data3d = dcmtools.get_pixel_array_from_sitk(image)
        # data3d, original_dtype = dcmreaddata.get_pixel_array_from_dcmobj(image)
        metadata = _metadata(image, datapath)
        return data3d, metadata

    @staticmethod
    def _fix_sitk_bug(path, metadata):
        """There is a bug in simple ITK for Z axis in 3D images. This is a fix.

        :param path: path to dicom file to read
        :param metadata: metadata to correct
        :return: corrected metadata
        """
        ds = dicom.read_file(path)
        try:
            metadata["voxelsize_mm"][0] = ds.SpacingBetweenSlices
        except Exception as e:
            logger.warning("Read dicom 'SpacingBetweenSlices' failed: ", e)
        return metadata

    # TODO remove this later
    # def read_hdf5(self, datapath):
    #     """Warning: Method is not implemented!
    #     Should read hdf5 data and return dictionary with data3d and metadata in itself.
    #
    #     :param datapath: path to hdf5 file
    #     :return: dict with data3d and metadata"""
    #     # TODO: implement this better, this is not working
    #     f = h5py.File(datapath, 'r')
    #     # import ipdb; ipdb.set_trace() #  noqa BREAKPOINT
    #     dd = self.__read_h5_key(f)
    #     # datap = {}
    #     # for item in f.attrs.keys():
    #     #     datap[item] = f.attrs['item']
    #     f.close()
    #     return dd

    # def __read_h5_key(self, grp):
    #     """Recursive function for reading h5 key.
    #     Used in "read_hdf5" method. Reads hdf5 file with h5py package and returns dictionary.
    #
    #     :param grp: hdf5 file to process
    #     :return: dict hopefully containing data3d and metadata aquired from hdf5 file
    #     """
    #     retval = {}
    #     for key in grp.keys():
    #         data = grp.get(key)
    #         if type(data) == h5py.Dataset:
    #             retval[key] = np.array(data)
    #         elif type(data) == h5py.Group:
    #             retval[key] = self.__read_h5_key(data)
    #         else:
    #             retval[key] = data
    #     return retval

    # noinspection PyPep8Naming
    @deprecated("Deprecated! Please, use get_overlay() function instead.")
    def GetOverlay(self):
        """Generates dictionary of ovelays"""
        return self.get_overlay()

    def get_overlay(self):
        """Generates dictionary of ovelays

        # NOTE: There is an obsolete method GetOverlay which is functionally identical to this one.
                As GetOverlay is deprecated, please do not use it - use this method instead.

        :return: dictionary of ovelays
        """
        if self.overlay_fcn is None:  # noqa
            return {}
        else:
            return self.overlay_fcn()


# NOTE(mareklovci - 2018_05_13): Should not this be dcmr.get_dcmdir_qt() instead? I would bet, dcmr.get_datapath_qt()
# does not exists
def get_datapath_qt(qt_app):
    # just call function from dcmr
    return dcmr.get_datapath_qt(qt_app)


def main():
    main_logger = logging.getLogger()

    main_logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    main_logger.addHandler(ch)

    # input parser
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('-i', '--inputfile',
                        default=None,
                        required=True,
                        help='input file')
    parser.add_argument('-sn', '--seriesnumber',
                        default=None,
                        help='seriesnumber')
    parser.add_argument('-d', '--debug',
                        action='store_true',
                        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)

    data3d, metadata = read(args.inputfile,
                            series_number=args.seriesnumber,
                            dataplus_format=False)
    import sed3
    ed = sed3.sed3(data3d)
    ed.show()


if __name__ == "__main__":
    main()



__all__ = ['dcmreaddata', 'datareader', 'datawriter', 'rawN', "datasets"]
__version__ = "1.19.0"
# from pycut import Model, ImageGraphCut
# from seed_editor_qt import QTSeedEditor
from .dcmreaddata import DicomReader
from .dcmreaddata import dicomdir_info
from .datawriter import DataWriter
from .datareader import DataReader
from .datareader import read
from .datawriter import write
from .datasets import download
from .fsbrowser import remove_if_exists
from imma import image_manipulation



#! /usr/bin/env python
# -*- coding: utf-8 -*-


import logging
logger = logging.getLogger(__name__)
import unittest
import numpy as np
import io3d.hdf5_io as hp

class Hdf5Test(unittest.TestCase):

    def test_read_write_h5(self):
        data = {'x': 'astring',
                'y': np.arange(10),
                'd': {'z': np.ones((2, 3)),
                      'b': b'bytestring'},
                "none": None,
                1: 1,
                }

        fn = "hdf5_testfile.h5"
        hp.save_dict_to_hdf5(data, fn)

        data2 = hp.load_dict_from_hdf5(fn)
        # this would raise an exception if there is something wrong
        np.testing.assert_equal(data, data2)
        self.assertEqual(data["x"], data2["x"])
        self.assertTrue(np.array_equal(data["y"], data2["y"]))
        self.assertEqual(data["d"]["b"], data2["d"]["b"])
        self.assertTrue(np.array_equal(data["d"]["z"], data2["d"]["z"]))
        self.assertEqual(data["none"], data2["none"])
        self.assertEqual(data[1], data2[1])






#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2015 mjirik <mjirik@hp-mjirik>
#
# Distributed under terms of the MIT license.

"""

"""

import logging
logger = logging.getLogger(__name__)
import argparse
import os.path as op
from . import misc


class CacheFile():
    def __init__(self, filename):
        """

        :param filename: File can be .yaml, .pkl or some other formats supported by misc.obj_from_file
        """
        self.filename = op.expanduser(filename)
        self.__update()

    def __update(self):
        if op.exists(self.filename):
            self.data = misc.obj_from_file(self.filename)
        else:
            self.data = {}

    def get(self, key):
        self.__update()
        return self.data[key]

    def get_or_none(self, key):
        self.__update()
        if key in self.data.keys():
            return self.data[key]
        else:
            return None

    def get_or_save_default(self, key, default_value):
        """
        Get value stored in cache file or store there default value.
        :param key:
        :param default_value:
        :return:
        """
        val = self.get_or_none(key)
        if val is None:
            self.update(key, default_value)
            val = default_value
        return val

    def update(self, key, value):
        self.data[key] = value
        misc.obj_to_file(self.data, self.filename)


def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    parser.add_argument(
        '-i', '--inputfile',
        default=None,
        required=True,
        help='input file'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)



if __name__ == "__main__":
    main()



import logging
logger = logging.getLogger(__name__)
import glob
import numpy as np
import os

from .import datareader

def remove_if_exists(filename):
    if os.path.exists(filename):
        os.remove(filename)

class FileSystemBrowser():
    def __init__(self, path=None):
        self.path = path
        self.preview_size = [100, 100]
        pass

    def get_path_info(self, path):
        """
        Get information about path (dir or file).
        :return:
        """

        # Fallowing function can be used for directory analysis
        # import io3d.dcmreaddata
        # dd = io3d.dcmreaddata.DicomDirectory(dirpath=path)
        # dd.get_stats_of_series_in_dir()

        # dd = dcmreaddata.DicomDirectory(self.path)
        # stats = dd.get_stats_of_series_in_dir()
        # studies_and_series = dd.get_stats_of_studies_and_series_in_dir()
        # import pydicom
        # pydicom.read_file(stats[7].dcmfilelist[0])

        # np.ndarray.resize()
        # JPG
        # import SimpleITK as Sitk

        # image = Sitk.ReadImage(datapath)
        # data3d = dcmtools.get_pixel_array_from_sitk(image)
        retval = {
            "name": "Study0545",
            "type": "dir",
            "preview": np.zeros(self.preview_size),
            "text": "1 study, 3 series, 18321 files, acquisition_date=2017-02-16 to 2017-02-19",
            "acquisition_date": ["2015-02-16", "2015-02-16"],
            "modality": "MRI",
            "path": "C:/data/Study0545"
        }
        return retval

    def get_dir_list(self):

        from . import dcmreaddata
        # datareader.read()
        # TODO check the design of output structure
        retval = [
            {
                "name": "Study0545",
                "type": "dir",
                "preview": np.zeros(self.preview_size),
                "text": "1 study, 3 series, 18321 files, acquisition_date=2017-02-16 to 2017-02-19",
                "acquisition_date": ["2015-02-16", "2015-02-16"],
                "modality": "MRI",
                "path": "C:/data/Study0545"
            },
            {
                "name": "Serie54864",
                "type": "serie",
                "preview": np.zeros(self.preview_size),
                "text": "3 series, 18321 files, acquisition_date=2017-02-16 to 2017-02-19",
                "acquisition_date": ["2015-02-16", "2015-02-16"],
                "modality": "MRI",
                "path": "c:/data/"

            },
            { # maybe signle file make no sense
                "name": "first.mhd",
                "type": "file",
                "preview": np.zeros(self.preview_size),
                "text": "[1x512x512], voxelsize_mm=[5.0, 0.5, 0.5], acquisition_date=2015-08-16",
                "voxelsize_mm": [5.0, 0.5, 0.5],
                "acquisition_date": "2015-08-16",
                "modality": "CT"

            },
        ]
        return retval
    # def file_anonymization(self, filename, output_filename=None):
    #     pass

    def recursive_anonymization(self, path, output_path=None):
        dirlist = glob.glob(path)

        pass
